program gbsarms, rclass
   version 12.1
   syntax anything [fweight pweight aweight iweight] [if] [in],  ///
   Grid(numlist ascending min = 3) ///  user specified grid of points
   [                               ///
   Lbound(real -1e20)              ///  lower bound for parameter
   Ubound(real 1e20)               ///  upper bound for parameter
   MAXtry(integer 50)              ///  maximum attempts to sample before giving up
   logp(real 1e20)                 ///  current log-posterior
   ]
   marksample touse
   
   tokenize "`anything'"
   local logfunct "`1'"
   local b "`2'"
   local ipar "`3'"
   if "`weight'" != "" local wt "[`weight'`exp']"
   
   if `logp' > 1e19    local logp = "."
   tempname slp
   
*-------------------------------
* Check grid specification
*-------------------------------   
   local n : word count `grid'
   tokenize `grid'
   if `lbound' >= `1' | `ubound' <= ``n'' {
      di as err "Bounds not consistent with grid"
      exit(499)
   }
   if `lbound' < -1e19 local lbound "."
   else if `b'[1,`ipar'] < `lbound' {
      di as err "Parameter `ipar': initial value outside the bounds"
	  exit(499)
   }
   if `ubound' > 1e19 local ubound "."
   else if `b'[1,`ipar'] > `ubound' {
      di as err "Parameter `ipar': initial value outside the bounds"
	  exit(499)
   }
*-------------------------------
* Call Mata for calculation
*-------------------------------   
   mata: gbsarms_stata("`logfunct' `slp' `b' `ipar' `wt' if `touse'","`slp'",`logp',"`b'",`ipar',"`grid'",`lbound',`ubound',`maxtry')
   return scalar try = `try'
   return scalar logp = `logp'
   return scalar fail = `fail'
end

