{smcl}
{* *! version 1.0.0 dec2012}{...}
{cmd:help gbsars} 
{hline}

{title:Title}

{p2colset 5 15 17 2}{...}
{p2col :{hi:gbsars} {hline 2}}Adaptive rejection sampling for use in Gibbs sampling{p_end}
{p2colreset}{...}


{title:Syntax}

{phang2}
{cmd:gbsars} {it:logpost rowvector parameter}
{ifin}
{weight}{cmd:,}
{opth g:rid(numlist)}
[{it:options}]

{phang}
{it:logpost} must be the name of the user's program for calculating the log of
the conditional posterior density.
See {help mhsnorm##userprog:Structure for the user's program} in
{helpb mhsnorm} for more details on how to write the program for the log
posterior.

{phang}
{it:rowvector} must be the name of the row vector containing the parameter
values.

{phang}
{it:parameter} must be the number of the parameter that is being updated.

{synoptset 16 tabbed}{...}
{synopthdr}
{synoptline}
{p2coldent:* {opth g:rid(numlist)}}initial grid of points{p_end}
{synopt:{opt l:bound(#)}}lower limit of range{p_end}
{synopt:{opt u:bound(#)}}upper limit of range{p_end}
{synopt:{opt max:try(#)}}maximum number of attempts to generate a new value{p_end}
{synoptline}
{p2colreset}{...}
{p 4 6 2}
* {cmd:grid()} is required.{p_end}
{p 4 6 2}
{opt fweight}s, {opt aweight}s, {opt iweight}s, and {opt pweight}s are
allowed; see {help weight}.


{title:Description}

{pstd}
{cmd:gbsars} uses adaptive rejection sampling (ARS) to simulate a value from a
one-dimensional posterior distribution. The posterior must be log concave, so
the algorithm is not completely general. {cmd:gbsars} is useful in Gibbs
sampling.  The grid consists of an initial set of at least 3 points placed
across the region where the posterior is thought to lie. Values can be
generated outside the range of this initial grid but not beyond the upper and
lower bounds.  The log posterior is evaluated at each of the points in the
grid, and those values are used to define an envelope enclosing the log
posterior. The program uses the envelope for rejection sampling, but any
rejected points are added to the grid to improve the fit of the envelope.  A
warning is printed if the posterior is found not to be log concave, and the
program will stop.  If the program is not log concave, then the more general
{helpb gbsarms} algorithm may be used, or the user might try slice sampling
with {helpb gbsslice} or a Metropolis-Hastings algorithm such as
{helpb mhsnorm}.  For speed of execution, the algorithm is programmed in Mata,
although the user's program for evaluating the log posterior may be in Stata
or Mata.  The Mata code is stored in the library {cmd:libmcmc}, which must
be installed for the command to work.{p_end}


{title:Options}

{phang}
{opth grid(numlist)} specifies a set of at least 3 ascending values defining
the initial grid for the envelope that encloses the log posterior.
{cmd:grid()} is required.{p_end}

{phang}
{opt lbound(#)} specifies the lower limit of the range of the posterior.
The default is minus infinity.{p_end}

{phang}
{opt ubound(#)} specifies the upper limit of the range of the posterior.
The default is infinity.{p_end}

{phang}
{opt maxtry(#)} specifies the number of rejection sampling attempts the
algorithm makes before giving up.  The default is {cmd:maxtry(50)}. If the
program takes more than a handful of attempts, then either the posterior is
extremely awkward or the initial grid was very poorly placed.{p_end}


{title:Stored results}

{pstd}
{cmd:gbsars} stores the following in {cmd:r()}:

{synoptset 12 tabbed}{...}
{p2col 5 12 16 2: Scalars}{p_end}
{synopt:{cmd:r(logp)}}log posterior at the new point{p_end}
{synopt:{cmd:r(try)}}number of evaluations of the log posterior{p_end}
{p2colreset}{...}


{title:Example}

{phang}
{cmd:. gbsars mylogpost b 3, grid(3 4 5 10) lbound(0)}{p_end}


{title:Author}

{pstd}
John Thompson{break}
Department of Health Sciences{break}
University of Leicester, UK{break}

{pstd}
Please report any errors to
{browse "mailto:john.thompson@le.ac.uk":john.thompson@le.ac.uk}.
{p_end}
