program logdensity
   version 12
   syntax anything [fweight] [if] [in]
   local distribution "`1'"
   local logp "`2'"
   capture confirm matrix `3'
   if !_rc {
      qui logdensity_`distribution' "M" `0'
   }
   else {
      capture confirm variable `3'
      if !_rc {
         marksample touse
         markout `touse' `3'
         tempname lnf
         qui logdensity_`distribution' "V"  `touse' `lnf' `0'
         qui su `lnf' [`weight'`exp'] if `touse' , meanonly
         scalar `logp' = `logp' + r(sum)
         local n = r(N)
         qui su `touse' [`weight'`exp'] if `touse' , meanonly
         if r(N) != `n' scalar `logp' = .
      }
      else {
         tempname slogp
         scalar `slogp' = `logp'
         qui logdensity_`distribution' "S" `0'
         scalar `logp' = `slogp' + `logp'
      }
   }
end

program logdensity_bernoulli
   local type "`1'"
   
   if "`type'" == "V" {
      local touse `2'
      local lnf `3'
      local y (`6')
      local p (`7')
      gen `lnf' = `y'*log(`p')+(1-`y')*log(1-`p') if `touse'
   }
   else if "`type'" == "S" {
      tempname y p
      local logp `3'
      scalar `y' = `4'
      scalar `p' = `5'
      scalar `logp' = (`y')*log(`p')+(1-`y')*log(1-`p')
   }
   else {
      di as err "No Bernoulli logdensity for data in a matrix"
      exit 198
   }
end

program logdensity_beta
   local type "`1'"
   
   if "`type'" == "V" {
      local touse `2'
      local lnf `3'
      local y (`6')
      local a (`7')
      local b (`8')
      gen `lnf' = (`a'-1)*log(`y') + (`b'-1)*log(1-`y')+lngamma(`a'+`b')- ///
         lngamma(`a')-lngamma(`b') if `touse'
   }
   else if "`type'" == "S" {
      tempname y a b
      local logp `3'
      scalar `y' = `4'
      scalar `a' = `5'
      scalar `b' = `6'
      scalar `logp' = (`a'-1)*log(`y') + (`b'-1)*log(1-`y')+ ///
         lngamma(`a'+`b')-lngamma(`a')-lngamma(`b')    }
   else {
      di as err "No beta logdensity for data in a matrix"
      exit 198
   }
end

program logdensity_binomial
   local type "`1'"
   
   if "`type'" == "V" {
      local touse `2'
      local lnf `3'
      local y (`6')
      local n (`7')
      local p (`8')
      gen `lnf' = `y'*log(`p')+(`n'-`y')*log(1-`p') if `touse'
   }
   else if "`type'" == "S" {
      tempname y n p
      local logp `3'
      scalar `y' = `4'
      scalar `n' = `5'
      scalar `p' = `6'
      scalar `logp' = `y'*log(`p')+(`n'-`y')*log(1-`p')
   }
   else {
      di as err "No binomial logdensity for data in a matrix"
      exit 198
   }
end

program logdensity_categorical
   local type "`1'"
   
   if "`type'" == "V" {
      local touse `2'
      local lnf `3'
      tempvar y
      gen `y' = `6'
      local p `7'
      gen `lnf' = log(`p'[`y']) if `touse'
   }
   else if "`type'" == "S" {
      tempname y
      scalar `y' = `3'
      local `4'
      scalar `logp' = log(`p'[`y'])
   }
   else {
      di as err "No categorical logdensity for data in a matrix"
      exit 198
   }
end

program logdensity_chisqr
   local type "`1'"
   
   if "`type'" == "V" {
      tempvar k
      local touse `2'
      local lnf `3'
      local y (`6')
      gen `k' = (`7')/2
      gen `lnf' = -`k'*0.69314718+(`k'-1)*log(`y')-`y'/2-lngamma(`k') if `touse'
   }
   else if "`type'" == "S" {
      tempname y k
      local logp `3'
      scalar `y' = `4'
      scalar `k' = (`5')/2
      scalar `logp' =  -`k'*0.69314718+(`k'-1)*log(`y')-`y'/2-lngamma(`k')
   }
   else {
      di as err "No chisqr logdensity for data in a matrix"
      exit 198
   }
end

program logdensity_dirichlet
   local type "`1'"
   
   if "`type'" == "V" {
      tempvar a
      local touse `2'
      local lnf `3'
      local y `6'
      gen `a' = `7' if `touse'
      su `a' if `touse', meanonly
      gen `lnf' = (`a'-1)*log(`y')-lngamma(`a')+ lngamma(r(sum))/r(N) if `touse'
   }
   else {
      di as err "No dirichlet logdensity for data in a scalar or matrix"
      exit 198
   }
end

program logdensity_exponential
   local type "`1'"
   
   if "`type'" == "V" {
      tempvar k
      local touse `2'
      local lnf `3'
      local y (`6')
      local mu (`7')
      gen `lnf' = -log(`mu')-`y'/`mu' if `touse'
   }
   else if "`type'" == "S" {
      tempname y mu
      local logp `3'
      scalar `y' = `4'
      scalar `mu' = `5'
      scalar `logp' =  -log(`mu')-`y'/`mu'
   }
   else {
      di as err "No exponential logdensity for data in a matrix"
      exit 198
   }
end

program logdensity_gamma
   local type "`1'"
   
   if "`type'" == "V" {
      tempvar yb
      local touse `2'
      local lnf `3'
      local y (`6')
      local a (`7')
      local b (`8')
      gen `yb' = `y'/`b' if `touse'
      gen `lnf' = -log(`b')+(`a'-1)*log(`yb')-`yb'-lngamma(`a') if `touse'
   }
   else if "`type'" == "S" {
      tempname y a b yb
      local logp `3'
      scalar `y' = `4'
      scalar `a' = `5'
      scalar `b' = `6'
      scalar `yb' = `y'/`b'
      scalar `logp' =  -log(`b')+(`a'-1)*log(`yb')-`yb'-lngamma(`a')
   }
   else {
      di as err "No gamma logdensity for data in a matrix"
      exit 198
   }
end

program logdensity_gengamma
   local type "`1'"
   
   if "`type'" == "V" {
      tempvar u sd
      local touse `2'
      local lnf `3'
      local y (`6')
      local a (`7')
      local b (`8')
      local g (`9')
      gen `yb' = `y'/`b' if `touse'
      gen `lnf' = log(`g')-log(`b')+(`a'-1)*log(`yb')-`yb'^`g'-lngamma(`a'/`g') if `touse'
   }
   else if "`type'" == "S" {
      tempname y a b g yb
      local logp `3'
      scalar `y' = `4'
      scalar `a' = `5'
      scalar `b' = `6'
      scalar `g' = `7'
      scalar `yb' = `y'/`b'
      scalar `logp' =  log(`g')-log(`b')+(`a'-1)*log(`yb')-`yb'^`g'-lngamma(`a'/`g')
   }
   else {
      di as err "No generalized gamma logdensity for data in a matrix"
      exit 198
   }
end

program logdensity_igamma
   local type "`1'"
   
   if "`type'" == "V" {
      local touse `2'
      local lnf `3'
      local y (`6')
      local a (`7')
      local b (`8')
      gen `lnf' = `a'*log(`b')+(-`a'-1)*log(`y')-`b'/`y'-lngamma(`a') if `touse'
   }
   else if "`type'" == "S" {
      tempname y a b
      local logp `3'
      scalar `y' = `4'
      scalar `a' = `5'
      scalar `b' = `6'
      scalar `logp' = `a'*log(`b')+(-`a'-1)*log(`y')-(`b'/`y')-lngamma(`a')
   }
   else {
      di as err "No inverse gamma logdensity for data in a matrix"
      exit 198
   }
end

program logdensity_igauss
   local type "`1'"
   
   if "`type'" == "V" {
      local touse `2'
      local lnf `3'
      local y (`6')
      local mu (`7')
      local lambda (`8')
      gen `lnf' = 0.5*log(`lambda'/(`y'*`y'*`y'))-0.5*(`lambda')*(`y'-`mu')*(`y'-`mu')/(`y'*`mu'*`mu') if `touse'
   }
   else if "`type'" == "S" {
      tempname y mu lambda
      local logp `3'
      scalar `y' = `4'
      scalar `mu' = `5'
      scalar `lambda' = `6'
      scalar `logp' =  0.5*log(`lambda'/(`y'*`y'*`y'))-0.5*`lambda'*(`y'-`mu')*(`y'-`mu')/(`y'*`mu'*`mu')
   }
   else {
      di as err "No inverse guassian logdensity for data in a matrix"
      exit 198
   }
end

program logdensity_laplace
   local type "`1'"
   
   if "`type'" == "V" {
      local touse `2'
      local lnf `3'
      local y (`6')
      local mu (`7')
      local phi (`8')
      gen `lnf' = -log(`phi')-abs(`y'-`mu')/`phi' if `touse'
   }
   else if "`type'" == "S" {
      tempname y mu phi
      local logp `3'
      scalar `y' = `4'
      scalar `mu' = `5'
      scalar `phi' = `6'
      scalar `logp' =  -log(`phi')-abs(`y'-`mu')/`phi'
   }
   else {
      di as err "No Laplace logdensity for data in a matrix"
      exit 198
   }
end

program logdensity_logistic
   local type "`1'"
   
   if "`type'" == "V" {
      tempvar u
      local touse `2'
      local lnf `3'
      local y (`6')
      local mu (`7')
      local b (`8')
      gen `u' = (`y'-`mu')/`b' if `touse'
      gen `lnf' =  -`u' - log(`b') - 2*log(1+exp(-`u')) if `touse'
   }
   else if "`type'" == "S" {
      tempname y mu b u
      local logp `3'
      scalar `y' = `4'
      scalar `mu' = `5'
      scalar `b' = `6'
      scalar `u' = (`y'-`mu') /`b'
      scalar `logp' =  -`u' - log(`b') - 2*log(1+exp(-`u'))
   }
   else {
      di as err "No logistic logdensity for data in a matrix"
      exit 198
   }
end

program logdensity_lognormal
   local type "`1'"
   
   if "`type'" == "V" {
      tempvar u
      local touse `2'
      local lnf `3'
      local y (`6')
      local mu (`7')
      local sd (`8')
      gen `u' = (log(`y')-`mu')/`sd' if `touse'
      gen `lnf' = -log(`sd')-0.5*`u'*`u'-`y' if `touse'
   }
   else if "`type'" == "S" {
      tempname y mu sd u
      local logp `3'
      scalar `y' = `4'
      scalar `mu' = `5'
      scalar `sd' = `6'
      scalar `u' = (log(`y')-`mu') /`sd'
      scalar `logp' = -log(`sd')-0.5*`u'*`u'-`y'
   }
   else {
      di as err "No lognormal logdensity for data in a matrix"
      exit 198
   }
end

program logdensity_mnormal
   local type "`1'"
   
   if "`type'" == "V" {
      tempvar r
      tempname S R T
      local touse `2'
      local lnf `3'
      local y (`6')
      local mu (`7')
      matrix S = `8'
      gen `r' = `y' - `mu' if `touse'
      mkmat `r' if `touse', mat(`R')
      matrix `T' = `R''*invsym(`S')*`R'
      local logp = -0.5*log(abs(det(`S')))-0.5*`T'[1,1]
      qui su `touse' if `touse'
      gen `lnf' = `logp'/r(N) if `touse'
   }
   else {
      di as err "No mnormal logdensity for data in a scalar or matrix"
      exit 198
   }
end

program logdensity_mt
   local type "`1'"
   
   if "`type'" == "V" {
      tempvar r
      tempname S R T
      local touse `2'
      local lnf `3'
      local y (`6')
      local mu (`7')
      matrix S = `8'
      local k (`9')
      local p = rowsof(`S')
      gen `r' = `y' - `mu' if `touse'
      mkmat `r' if `touse', mat(`R')
      matrix `T' = `R''*invsym(`S')*`R'
      local logp = -0.5*log(abs(det(`S'))) -0.5*(`k'+`p')*log(1+`T'[1,1]/`k')+lngamma(0.5*(`k'+`p'))-lngamma(0.5*`k')
      qui su `touse' if `touse'
      gen `lnf' = `logp'/r(N) if `touse'
   }
   else {
      di as err "No mt logdensity for data in a scalar or matrix"
      exit 198
   }
end

program logdensity_multinomial
   local type "`1'"
   
   if "`type'" == "V" {
      local touse `2'
      local lnf `3'
      local y (`6')
      local p (`7')
      gen `lnf' = `y'*log(`p') if `touse'
   }
   else {
      di as err "No multinomial logdensity for data in a scalar or matrix"
      exit 198
   }
end

program logdensity_negbin
   local type "`1'"
   
   if "`type'" == "V" {
      local touse `2'
      local lnf `3'
      local y (`6')
      local n (`7')
      local p (`8')
      local r (`9')
      gen `lnf' = `r'*log(`p')+`y'*log(1-`p')+lngamma(`y'+`r')- ///
         lngamma(`r')-lngamma(`y'+1) if `touse'
   }
   else if "`type'" == "S" {
      tempname y n p r
      local logp `3'
      scalar `y' = `4'
      scalar `n' = `5'
      scalar `p' = `6'
      scalar `r' = `7'
      scalar `logp' =  `r'*log(`p')+`y'*log(1-`p')+ ///
         lngamma(`y'+`r')-lngamma(`r')+lngamma(`y'+1)
   }
   else {
      di as err "No negbin logdensity for data in a matrix"
      exit 198
   }
end

program logdensity_normal
   local type "`1'"
   
   if "`type'" == "V" {
      tempvar u
      local touse `2'
      local lnf `3'
      local y (`6')
      local mu (`7')
      local sd (`8')
      gen `u' = (`y'-`mu')/`sd' if `touse'
      gen `lnf' = -log(`sd') - 0.5*`u'*`u' if `touse'
   }
   else if "`type'" == "S" {
      tempname y mu sd u
      local logp `3'
      scalar `y' = `4'
      scalar `mu' = `5'
      scalar `sd' = `6'
      scalar `u' = (`y'-`mu') /`sd'
      scalar `logp' = - log(`sd') - 0.5*`u'*`u'
   }
   else {
      di as err "No normal logdensity for data in a matrix"
      exit 198
   }
end

program logdensity_pareto
   local type "`1'"
   
   if "`type'" == "V" {
      local touse `2'
      local lnf `3'
      local y (`6')
      local a (`7')
      local c (`8')
      gen `lnf' = log(`a')+`a'*log(`c')-(`a'+1)*log(`y') if `touse'
   }
   else if "`type'" == "S" {
      tempname y a c
      local logp `3'
      scalar `y' = `4'
      scalar `a' = `5'
      scalar `c' = `6'
      scalar `logp' = `logp' +log(`a')+`a'*log(`c')-(`a'+1)*log(`y')
   }
   else {
      di as err "No pareto logdensity for data in a matrix"
      exit 198
   }
end

program logdensity_poisson
   local type "`1'"
   
   if "`type'" == "V" {
      local touse `2'
      local lnf `3'
      local y (`6')
      local mu (`7')
      gen `lnf' = -`mu' + `y'*log(`mu') if `touse'
   }
   else if "`type'" == "S" {
      tempname y mu
      local logp `3'
      scalar `y' = `4'
      scalar `mu' = `5'
      scalar `logp' =  - `mu' + `y'*log(`mu') - lngamma(`y'+1)
   }
   else {
      di as err "No pareto logdensity for data in a matrix"
      exit 198
   }
end

program logdensity_t
   local type "`1'"
   
   if "`type'" == "V" {
      tempvar u
      local touse `2'
      local lnf `3'
      local y (`6')
      local mu (`7')
      local sd (`8')
      local k (`9')
      gen `u' = (`y'-`mu')/`sd' if `touse'
      gen `lnf' = -log(`sd')-((`k'+1)/2)*log(1+`u'*`u'/(`k')) if `touse'
   }
   else if "`type'" == "S" {
      tempname y mu sd u
      local logp `3'
      scalar `y' = `4'
      scalar `mu' = `5'
      scalar `sd' = `6'
      scalar `k' = `7'
      scalar `u' = (`y'-`mu') /`sd'
      scalar `logp' = -log(`sd')-((`k'+1)/2)*log(1+`u'*`u'/`k')
   }
   else {
      di as err "No t logdensity for data in a matrix"
      exit 198
   }
end

program logdensity_uniform
   local type "`1'"
   
   if "`type'" == "V" {
      local touse `2'
      local lnf `3'
      local y (`6')
      local a (`7')
      local b (`8')
      gen `lnf' = -log(`b'-`a')/inrange(`y',`a',`b') if `touse'
   }
   else if "`type'" == "S" {
      tempname y a b
      local logp `3'
      scalar `y' = `4'
      scalar `a' = `5'
      scalar `b' = `6'
      scalar `logp' = - log(`b'-`a')/inrange(`y',`a',`b')
   }
   else {
      di as err "No uniform logdensity for data in a matrix"
      exit 198
   }
end

program logdensity_weibull
   local type "`1'"
   
   if "`type'" == "V" {
      tempvar yb
      local touse `2'
      local lnf `3'
      local y (`6')
      local v (`7')
      local mu (`8')
      gen `yb' = `y'/`mu' if `touse'
      gen `lnf' =  log(`v'/`mu')+(`v'-1)*log(`yb')-`yb'^`v' if `touse'
   }
   else if "`type'" == "S" {
      tempname y v mu yb
      local logp `3'
      scalar `y' = `4'
      scalar `v' = `5'
      scalar `mu' = `6'
      scalar `yb' = `y'/`mu'
      scalar `logp' =  log(`v'/`mu')+(`v'-1)*log(`yb')-`yb'^`v'
   }
   else {
      di as err "No weibull logdensity for data in a matrix"
      exit 198
   }
end

program logdensity_wishart
   local type "`1'"
   
   if "`type'" == "M" {
      tempname Y S T
      matrix `Y' = `3'
      matrix `S' = `4'
      local k = `5'
      local p = rowsof(`Y')
      matrix `T' = invsym(`S')*`Y'
      scalar `logp' = -0.5*trace(`T')+ ///
      0.5*(`k'-`p'-1)*log(abs(det(`Y')))-0.5*(`k')*log(abs(det(`S')))
   }
   else {
      di as err "For the wishart logdensity the data must be in a matrix"
      exit 198
   }
end

