*============================================================================
* mhsmnorm: Metropolis-Hastings Sampler based on a multivariate-normal proposal distribution
* Author:   John Thompson
* Date:     Dec 2012
*============================================================================

program mhsmnorm, rclass
   version 12.1
   syntax anything  [fweight] [if] [in], ///
   Cholesky(string)    ///
   [                   ///
   LOGP(real 1e20)     ///
   DEBUG               ///
   ]

   tokenize "`anything'"
   local logpost "`1'"
   local b "`2'"
   local ipar1 = `3'
   local ipar2 = `ipar1' - 1 + rowsof(`cholesky')
   
   tempname logl newlogl oldb
   if "`debug'" == "" local qui "qui"
   
   matrix `oldb' = `b'
   scalar `logl' = .
   if `logp' < 1e19 scalar `logl' = `logp'
   if `logl' == .  `qui' `logpost' `logl' `b' `ipar1'  [`weight'`exp'] `if' `in'
   if `logl' == .  {
      `qui' `logpost' `logl' `b' `ipar' [`weight'`exp'] `if' `in'
      if `logl' == .  {
         di as err "mhsmnorm: cannot calculate log-posterior for parameters `ipar1'-`ipar2'"
         matrix list `b'
         exit(1400)
      }
   }
   
   forvalues p = `ipar1'/`ipar2' {
      local i = `p' - `ipar1' + 1
      local u`i' = rnormal()
      local z = 0
      forvalues j = 1/`i' {
         local z = `z' + `cholesky'[`i',`j']*`u`j''
      }
      matrix `b'[1,`p'] = `b'[1,`p'] + `z'
   }
   
   scalar `newlogl' = .
   `qui' `logpost' `newlogl' `b'  [`weight'`exp'] `if' `in'
   if `newlogl' == .  {
      di as err "mhsmnorm: cannot calculate log-posterior for parameters `ipar1'-`ipar2'"
      matrix list `b'
      exit(1400)
   }
   
   
   if log(uniform()) < (`newlogl'-`logl')  & `newlogl' != . local accept = 1
   else {
      local accept = 0
      matrix `b' = `oldb'
      scalar `newlogl' = `logl'
   }
   return scalar accept = `accept'
   return scalar logp = `newlogl'
end
