*============================================================================
* mhsnorm: Metropolis-Hastings Sampler based on a normal proposal distribution
* Author:  John Thompson
* Date:    Dec 2012
*============================================================================

program mhsnorm, rclass
   version 12.1   
   syntax anything  [fweight] [if] [in], ///
   SD(string)        ///
   [                 ///
   LOGP(real 1e20)   /// 
   DEBUG             ///
   ]

   tokenize "`anything'"
   local logpost  "`1'"
   local b        "`2'"
   local ipar    = `3'
   
   tempname logl newlogl
   if "`debug'" == "" local qui "qui"
   
   local oldvalue = `b'[1,`ipar']
   scalar `logl' = .
   if `logp' < 1e19 scalar `logl' = `logp'
   if `logl' == .  {
      `qui' `logpost' `logl' `b' `ipar' [`weight'`exp'] `if' `in'
      if `logl' == .  {
         di as err "mhsnorm: cannot calculate log-posterior for parameter `ipar'"
         matrix list `b'
         exit(1400)
      }
   }
   
   matrix `b'[1,`ipar'] = `oldvalue' + `sd'*rnormal()
   
   scalar `newlogl' = .
   `qui' `logpost' `newlogl' `b' `ipar' [`weight'`exp'] `if' `in'
   if `newlogl' == .  {
      di as err "mhsnorm: cannot calculate log-posterior for parameter `ipar'"
      matrix list `b'
      exit(1400)
   }
   
   if log(uniform()) < (`newlogl'-`logl')  local accept = 1
   else {
      local accept = 0
      matrix `b'[1,`ipar'] = `oldvalue'
      scalar `newlogl' = `logl'
   }
   return scalar accept = `accept'
   return scalar logp = `newlogl'
end
