*------------------------------
* Code for Chapter 10
* working directory must include the data file: 
*------------------------------
clear
set more off
set scheme sj
program drop _all
pause off

*------------------------------------
* Neural tube example
* Bayes Factor by direct simulation
*------------------------------------
clear
set obs 500000
set seed 546109
gen p1 = rgamma(25,1)
gen p2 = rgamma(20,1)
gen p3 = rgamma( 5,1)
gen sp = p1+p2+p3
replace p1 = p1/sp
replace p2 = p2/sp
replace p3 = p3/sp
gen theta = rnormal(0.7,0.5)

gen logp = 439*log(p1)+326*log(p2)+83*log(p3)
gen OR3 = exp(theta)
* Model M1
gen logp1 = logp + 151*log(p1)+171*log(p2)+73*log(p3)

* Model M2
gen OR2 = 1
gen sq = p1+OR2*p2+OR3*p3
gen q1 = p1/sq
gen q2 = OR2*p2/sq
gen q3 = OR3*p3/sq
gen logp2 = logp + 151*log(q1)+171*log(q2)+73*log(q3)

* Model M2
replace OR2 = exp(theta/2)
replace sq = p1+OR2*p2+OR3*p3
replace q1 = p1/sq
replace q2 = OR2*p2/sq
replace q3 = OR3*p3/sq
gen logp3 = logp + 151*log(q1)+171*log(q2)+73*log(q3)

* Model M3
replace OR2 = OR3
replace sq = p1+OR2*p2+OR3*p3
replace q1 = p1/sq
replace q2 = OR2*p2/sq
replace q3 = OR3*p3/sq
gen logp4 = logp + 151*log(q1)+171*log(q2)+73*log(q3)

* Baseline for exponentiation
summarize logp3
local m = r(max)
* Marginal Likelihoods
forvalues i=1/4 {
   gen m`i' = exp(logp`i'-`m')
   summarize m`i'
   local m`i' = r(mean)
}
display "BF31" %10.0f `m3'/`m1'
display "BF32" %10.0f `m3'/`m2'
display "BF34" %10.0f `m3'/`m4'


pause "Press q to continue"

*-----------------------------------
* Priors in WinBUGS
*-----------------------------------
tomodel chapter10.do temp/priormodel.txt
/*
model {
   p[1:3] ~ ddirich(a[])
   theta ~ dnorm(0.7,4)
}
*/
wbslist ( a=c(25,20,5) ) using temp/data.txt, replace
wbslist ( p=c(0.5,0.4,0.1), theta=0.7 ) using temp/init.txt, replace

wbsscript using temp/script.txt, ///
   model(priormodel.txt) data(data.txt) init(init.txt) ///
   burn(0) update(50000) thin(5) set(p theta) coda(BF) ///
   replace openbugs path(`c(pwd)'/temp)
wbsrun using `c(pwd)'/temp/script.txt, openbugs
wbscoda using temp/BF, clear openbugs

*-----------------------------------
* Harminic Mean using OpenBUGS
*-----------------------------------
tomodel chapter10.do temp/modelBF1.txt
/*
model {
    p[1:3] ~ ddirich(a[])
    theta <- 0
   for (i in 1:3) {
      q[i] <- p[i]
   }
    x[1:3] ~ dmulti(p[1:3],Nx)
    y[1:3] ~ dmulti(p[1:3],Ny)
}
*/
tomodel chapter10.do temp/modelBF2.txt
/*
model {
    p[1:3] ~ ddirich(a[])
    theta ~ dnorm(0.7,4)
    OR3 <- exp(theta)
    OR2 <- 1
    s <- p[1] + OR2*p[2] + OR3*p[3]
    q[1] <- p[1]/s
    q[2] <- OR2*p[2]/s
    q[3] <- OR3*p[3]/s
    x[1:3] ~ dmulti(p[],Nx)
    y[1:3] ~ dmulti(q[],Ny)
}
*/
tomodel chapter10.do temp/modelBF3.txt
/*
model {
    p[1:3] ~ ddirich(a[])
    theta ~ dnorm(0.7,4)
    OR3 <- exp(theta)
    OR2 <- exp(theta/2)
    s <- p[1] + OR2*p[2] + OR3*p[3]
    q[1] <- p[1]/s
    q[2] <- OR2*p[2]/s
    q[3] <- OR3*p[3]/s
    x[1:3] ~ dmulti(p[],Nx)
    y[1:3] ~ dmulti(q[],Ny)
}
*/
tomodel chapter10.do temp/modelBF4.txt
/*
model {
    p[1:3] ~ ddirich(a[])
    theta ~ dnorm(0.7,4)
    OR3 <- exp(theta)
    OR2 <- OR3
    s <- p[1] + OR2*p[2] + OR3*p[3]
    q[1] <- p[1]/s
    q[2] <- OR2*p[2]/s
    q[3] <- OR3*p[3]/s
    x[1:3] ~ dmulti(p[],Nx)
    y[1:3] ~ dmulti(q[],Ny)
}
*/
* DATA FILE - dataBF2.txt
* wbslist ( a=c(25,20,5),x=c(439,326,83), Nx=848, y=c(151,171,73), Ny=395 ) using temp/dataBF2.txt, replace
* INITS FILE - initBF2.txt
* wbslist ( p=c(0.5,0.4,0.1), theta = 0.7) using initBF2.txt, replace
*---------------------------------------------
* Run each model in openbugs
*---------------------------------------------
clear
set more off
wbslist (a=c(25,20,5), x=c(439,326,83), Nx=848, y=c(151,171,73), Ny=395) ///
   using temp/dataBF.txt, replace
wbslist (p=c(0.5,0.4,0.1), theta = 0.7) using temp/initBF.txt, replace
forvalues M=1/4 {
   wbsscript using temp/script.txt,  ///
     model(modelBF`M'.txt) data(dataBF.txt) init(initBF.txt)  ///
     burn(10000) update(50000) set(p q) coda(BF) replace openbugs path(`c(pwd)'/temp)
   wbsrun using `c(pwd)'/temp/script.txt, openbugs time 
   wbscoda using temp/BF, clear openbugs
   gen logp`M' = 439*log(p_1)+326*log(p_2)+83*log(p_3) + ///
                 151*log(q_1)+171*log(q_2)+73*log(q_3)
   keep logp`M'
   if `M' > 1 qui merge 1:1 _n using temp/BFH.dta, nogen
   save temp/BFH.dta, replace
}

use temp/BFH.dta, clear
summarize logp3
local m = r(max)
gen m1 = 1/exp(logp1-`m')
gen m2 = 1/exp(logp2-`m')
gen m3 = 1/exp(logp3-`m')
gen m4 = 1/exp(logp4-`m')
summarize m1  
local m1 = 1/r(mean)
summarize m2  
local m2 = 1/r(mean)
summarize m3 
local m3 = 1/r(mean)
summarize m4  
local m4 = 1/r(mean)
display %10.0f `m3'/`m1' %10.0f `m3'/`m2' %10.0f `m3'/`m4'

pause "Press q to continue"

*-----------------------------------
* BF using Chib method
*-----------------------------------
tomodel chapter10.do temp/modelCB.txt
/* 
model {
    M ~ dcat(alpha[1:4])
    p[1:3] ~ ddirich(a[1:3])

    m2[1] <- 0.75 m2[2] <- 0.7  m2[3] <- 0.75 m2[4] <- 0.75
    t2[1] <- 25   t2[2] <- 4    t2[3] <- 25   t2[4] <- 25

    m3[1] <- 0.9  m3[2] <- 0.9  m3[3] <- 0.7  m3[4] <- 0.9
    t3[1] <- 25   t3[2] <- 25   t3[3] <- 4    t3[4] <- 25

    m4[1] <- 0.6  m4[2] <- 0.6  m4[3] <- 0.6  m4[4] <- 0.7
    t4[1] <- 25   t4[2] <- 25   t4[3] <- 25   t4[4] <- 4

    theta[1,1] <- 0
    theta[1,2] <- 0

    theta[2,1] <- 0
    theta[2,2] ~ dnorm(m2[M],t2[M])

    theta[3,2] ~ dnorm(m3[M],t3[M])
    theta[3,1] <- theta[3,2]/2

    theta[4,2] ~ dnorm(m4[M],t4[M])
    theta[4,1] <- theta[4,2]

    
    OR2 <- exp(theta[M,1])
    OR3 <- exp(theta[M,2])

    s <- p[1] + OR2*p[2] + OR3*p[3]
    q[1] <- p[1]/s
    q[2] <- OR2*p[2]/s
    q[3] <- OR3*p[3]/s

    x[1:3] ~ dmulti(p[1:3],Nx)
    y[1:3] ~ dmulti(q[1:3],Ny)
}
*/

wbslist (alpha=c(0.9979,0.001,0.0001,0.001), x=c(439,326,83), ///
    y=c(151,171,73), a=c(25,20,5), Nx=848, Ny=395) using temp/dataCB.txt, replace
matrix input theta = (. . \ . 0.7 \ . 0.7 \ . 0.7)
matrix input p = (0.4, 0.4, 0.2)
wbslist(matrix theta p) (M=1) using temp/initCB.txt, replace
wbsscript using temp/script.txt,                             ///
   model(modelCB.txt) data(dataCB.txt) init(initCB.txt) ///
   burn(0) update(100000) thin(5) log(logCB.txt)        ///
   set(p theta M) coda(CB) openbugs replace path(`c(pwd)'/temp) 
wbsrun using `c(pwd)'/temp/script.txt , openbugs
type temp/logCB.txt
wbscoda using temp/CB, clear openbugs
forvalues i=1/4 {
   count if M == `i'
   local n`i' = r(N)
}
display (`n3'/`n1')/(0.0001/0.9979)
display (`n3'/`n2')/(0.0001/0.001)
display (`n3'/`n4')/(0.0001/0.001)

pause "Press q to continue"

*-----------------------------------------
* Model averaging
* As before but with different priors
*-----------------------------------------
wbslist (alpha=c(0.05,0.05,0.05,0.85), x=c(439,326,83), y=c(151,171,73), a=c(25,20,5) ,Nx=848, Ny=395) ///
   using temp/dataCB.txt, replace
wbslist(theta=structure(.Data=c(NA,NA,NA,0.7,NA,0.7,NA,0.7),.Dim=c(4,2)), ///
   p=c(0.4,0.4,0.2),M=1) using temp/initCB.txt, replace
wbsscript using temp/script.txt, ///
     model(modelCB.txt) data(dataCB.txt) init(initCB.txt) ///
     burn(0) update(500000) log(logCB.txt) ///
     set(p theta M) coda(CB) openbugs replace path(`c(pwd)'/temp) 
wbsrun using `c(pwd)'/temp/script.txt, openbugs
type temp/logCB.txt
wbscoda using temp/CB, clear openbugs
gen logor = 0
replace logor = theta_2_2 if M == 2
replace logor = theta_3_2 if M == 3
replace logor = theta_4_2 if M == 4
mcmcstats logor
mcmcstats logor if M == 2
mcmcstats logor if M == 3
mcmcstats logor if M == 4

qui count if M == 3
local A = r(N)/500000
kdensity logor if M==3, gen(x3 y3)
replace y3 = y3*`A'
qui count if M == 4
local A = r(N)/500000
kdensity logor if M==4, gen(x4 y4)
replace y4 = y4*`A'
kdensity logor, gen(x y)
twoway ( line y x, lpat(solid)) (line y3 x3, lpat(dash)) ///
   ( line y4 x4, lpat(dash)), leg(off) xtitle(logOR (TT vs CC)) yscale(off)
* FIGURE 10.1
graph export eps\chap10_ma.eps, replace

pause "Press q to continue"

*-----------------------------------------
* DIC for each model
* same model etc as for BF
*-----------------------------------------
forvalues M=1/4 {
qui wbsscript using temp/script.txt,  ///
     model(modelBF`M'.txt) data(dataBF.txt) init(initBF.txt)  ///
     burn(10000) update(50000) replace dic openbugs log(log.txt) path(`c(pwd)'/temp)
qui wbsrun using `c(pwd)'/temp/script.txt, openbugs 
type temp/log.txt
}

pause "Press q to continue"
