*------------------------------
* Code for Chapter 4
* working directory must include the data file: data\shoulder.dta
*------------------------------
clear
set scheme sj
program drop _all
set more off
pause off
*-------------------------------
* Section 4.2
* Shoulder pain gibbs sampler
* figure 4.2
*-------------------------------
use data/shoulder.dta, clear
gen y = log(pain/(10-pain) )
* FIGURE 4.1  
scatter y age , ytitle(Transformed pain score) xtitle(Age (years))
graph export eps/shoulder.eps, replace

pause "Press q to continue"
*================================================================================

*-------------------------------
* Gibbs sampler
*-------------------------------
set seed 569014
use data/shoulder.dta, clear
gen y = log(pain/(10-pain) )
reg y age
gen x = age - 55 
*-------------------------------
* Summary Statistics
*-------------------------------
summarize y
scalar Sy = r(sum)
summarize x
scalar Sx = r(sum)
gen yy = y*y
summarize yy
scalar Syy = r(sum)
gen xx = x*x
summarize xx
scalar Sxx = r(sum)
gen xy = x*y
summarize xy
scalar Sxy = r(sum)
*-------------------------------
* Gibbs sampler updates
*-------------------------------
tempname pf
postfile `pf' alpha beta tau using temp/temp.dta,replace
scalar alpha = 0
scalar beta = 0
scalar tau = 1
forvalues i=1/2500 {
  post `pf' (alpha) (beta) (tau)
  scalar b = 0.5*(8+Syy+24*alpha*alpha+beta*beta*Sxx- ///
     2*alpha*Sy-2*beta*Sxy+2*alpha*beta*Sx)
  scalar tau = rgamma(16,1/b)
  scalar t = 0.01+24*tau
  scalar m = (tau*(Sy-beta*Sx))/t
  scalar alpha = rnormal(m,1/sqrt(t))
  scalar t = 100+tau*Sxx
  scalar m = (tau*(Sxy-alpha*Sx))/t
  scalar beta = rnormal(m,1/sqrt(t))
}
postclose `pf'
use temp/temp.dta, clear
gen sd = 1/sqrt(tau)
replace alpha = alpha - 55*beta
mcmcstats alpha beta tau sd if _n > 500
* FIGURE 4.1  
mcmctrace alpha beta tau sd
graph export eps/shouldertrace.eps, replace

pause "Press q to continue"
*================================================================================

*-------------------------------
* Section 4.2
* Shoulder pain using mcmc
*-------------------------------
program drop _all
use data/shoulder.dta, clear
gen y = log(pain/(10-pain) )
gen x = age - 55 
summarize y
scalar Sy = r(sum)
summarize x
scalar Sx = r(sum)
gen yy = y*y
summarize yy
scalar Syy = r(sum)
gen xx = x*x
summarize xx
scalar Sxx = r(sum)
gen xy = x*y
summarize xy
scalar Sxy = r(sum)

program drop _all 
program myGibb
   args dummy b ipar
  
   local alpha = `b'[1,1]
   local beta  = `b'[1,2]
   local tau   = `b'[1,3]
   if `ipar' == 1 {
      local t = 0.01+24*`tau'
      local m = (`tau'*(Sy-`beta'*Sx))/`t'
      matrix `b'[1,1] = `m' + rnormal()/sqrt(`t')
   }
   else if `ipar' == 2 {
      local t = 100+`tau'*Sxx
      local m = (`tau'*(Sxy-`alpha'*Sx))/`t'
      matrix `b'[1,2] = `m' + rnormal()/sqrt(`t')
   }
   else {
      local a1 = 16
      local a2 = 0.5*(8+Syy + 24*`alpha'*`alpha' + `beta'*`beta'*Sxx ///
        - 2*`alpha'*Sy - 2*`beta'*Sxy + 2*`alpha'*`beta'*Sx)
      matrix `b'[1,3] = rgamma(`a1',1/`a2')
   }
end   
matrix b = ( 0, 0, 1 )
mcmcrun dummy b using temp/temp.csv, samp(3(myGibb)) ///
  burnin(500) updates(2000) par(alpha beta tau) replace
import delimited temp/temp.csv, clear
gen sd = 1/sqrt(tau)
replace alpha = alpha - 55*beta

mcmcstats alpha beta tau sd 

pause "Press q to continue"
*================================================================================

*-------------------------------
* Figure 4.3
* Diagram to illustrate Griddy sampling
*-------------------------------
clear
set obs 7
gen x = -4 + _n
gen y = 0.7*normalden(x,0,1)+0.3*normalden(x,1,0.5)

twoway (function y=0.7*normalden(x,0,1)+0.3*normalden(x,1,0.5) , range(-3.5 4)  lpat(solid)) ///
  (scatter y x , msym(circle) ) (line y x, lpat(dash) ), leg(off) xtitle(theta) ///
  ytitle("") ylabel(none) nodraw
graph save temp/plot1.gph, replace
twoway (bar y x, lpat(dash) col(gs12)) (function y=0.7*normalden(x,0,1)+0.3*normalden(x,1,0.5) , ///
   range(-3.5 4) lpat(solid)) ///
  (scatter y x , msym(circle) )   , ///
  leg(off) xtitle(theta) ytitle("") ylabel(none) nodraw
graph save temp/plot2.gph, replace
* FIGURE 4.3
graph combine temp/plot2.gph temp/plot1.gph 
graph export eps/chap4_griddy.eps, replace

pause "Press q to continue"
*================================================================================

*-------------------------------
* Figure 4.4
* Diagram to illustrate Slice sampling
*-------------------------------
clear
range x 0 10 1000
gen y = gammaden(8,0.5,0,x)
twoway area y x , fcol(gs12) yscale(off) xtitle(theta)
graph save temp/plot1.gph, replace 
* FIGURE 4.4
graph export eps/slice1.eps, replace
qui gen xs = .
qui gen ys = .
qui replace xs = 2.45 in 1
qui replace xs = 4.8 in 2
qui replace ys = 0.203 in 1/2
local ysd = ys[1] - 0.006
twoway (line y x ) (area ys xs, base(`ysd') col(gs12) ) , ///
  xtitle(theta) xline(2.8, lpat(dash)) ///
  yline(0.2, lpat(dash) ) yscale(off) legend(off)
* FIGURE 4.4
graph export eps/slice2.eps, replace

graph save temp/plot2.gph, replace
graph combine temp/plot1.gph temp/plot2.gph 
pause "Press q to continue"
*================================================================================

*-------------------------------
* Figure 4.5
* Diagram to illustrate Rejection sampling
*-------------------------------
clear
range x 0 10 1000
gen yl = 0.8*normalden(x,6,1)+0.2*normalden(x,4,0.5)
gen yu = 5*normalden(x,5,2)
qui gen xs = .
qui replace xs = 5.5 in 1
qui replace xs = 5.7 in 2
qui gen ysl = 0.8*normalden(xs,6,1)+0.2*normalden(xs,4,0.5)
qui gen ysu = 5*normalden(xs,5,2)
twoway (area ysu xs, fcol(gs14)) (area ysl xs, fcol(gs9)) ///
  (line yl x, lpat(solid)) (line yu x , lpat(solid)) ///
  , legend(off) yscale(off) xtitle(theta)
* FIGURE 4.5
graph export eps/rs.eps, replace
pause "Press q to continue"
*================================================================================

*-------------------------------
* Figure 4.6
* Diagram to illustrate ARMS
*-------------------------------
clear
range x  0 10 1001
gen y = log(normalden((log(x)-log(4))/0.3,0,1))
drop if x < 1 | y < -3
local x1 = 2.5
local x2 = 3.5
local x3 = 5
local x4 = 7
gen yp = .
gen xp = .
forvalues i=1/4 {
   replace xp = `x`i'' in `i'
   replace yp = log(normalden((log(`x`i'')-log(4))/0.3,0,1)) in `i'
}
forvalues i=1/3 {
local j = `i' + 1
local b`i' = (yp[`j']-yp[`i'])/(xp[`j']-xp[`i'])
local a`i' = yp[`i'] - `b`i''*xp[`i']
}
local xc = (`a3'-`a1')/(`b1'-`b3')
local yc = `a1'+`b1'*`xc'
local xdash = 4.25
local ydash = log(normalden((log(`xdash')-log(4))/0.3,0,1))
twoway (line y x ) (scatter yp xp , msize(large) msym(circle) ) ///
 (function y=`a1'+`b1'*x, range(`x1' `x2') lpat(dash) ) ///
 (function y=`a2'+`b2'*x, range(`x2' `x3') lpat(dash) ) ///
 (function y=`a3'+`b3'*x, range(`x3' `x4') lpat(dash) ) ///
 (function y=`a1'+`b1'*x, range(1.5 `x1') lpat(solid) lwidth(thick)) ///
 (function y=`a1'+`b1'*x, range(`x2' 4.5) lpat(solid) ) ///
 (function y=`a2'+`b2'*x, range(2 `x2') lpat(solid) ) ///
 (function y=`a2'+`b2'*x, range(`x3' 8) lpat(solid) ) ///
 (function y=`a3'+`b3'*x, range(`x2' `x3') lpat(solid) ) ///
 (function y=`a3'+`b3'*x, range(`x4' 8) lpat(solid) lwidth(thick)) ///
 (function y=`a1'+`b1'*x, range(`x2' `xc') lpat(solid) lwidth(thick) ) ///
 (function y=`a3'+`b3'*x, range(`xc' `x3') lpat(solid) lwidth(thick) ) ///
 (function y=`a2'+`b2'*x, range(`x1' `x2') lpat(solid) lwidth(thick) ) ///
 (function y=`a2'+`b2'*x, range(`x3' `x4') lpat(solid) lwidth(thick) ) ///
 , leg(off) ylabel(none) ytitle(log posterior) xtitle(parameter) ///
  xscale(range(1 8) ) xlabel( 1(2)7 ) xline(`x1',lpat(dash_dot)) xline(`x4',lpat(dash_dot)) 
* FIGURE 4.6
graph export eps/arms1.eps, replace
pause "Press q to continue"
*================================================================================

*-------------------------------
* Section 4.6
* Laplace regression
* Inefficient logpost
*-------------------------------
program drop _all
use data/shoulder.dta, clear
gen y = log(pain/(10-pain) )
gen x = age - 55 
set seed 456932
*-------------------------------
* inefficient log-posterior
*-------------------------------
program logpost
    args logp b

    local alpha = `b'[1,1]
    local beta  = `b'[1,2]
    local sigma = `b'[1,3]
    local phi   = `sigma'/1.414214
    scalar `logp' = 0
    logdensity laplace `logp' y `alpha'+`beta'*x `phi'
    logdensity normal   `logp' `alpha' 0 10
    logdensity normal   `logp' `beta'  0 0.1 
    logdensity uniform  `logp' `sigma' 0 5	
end

*-------------------------------
* mcmcrun with griddy
*-------------------------------
matrix b = (1,0,1)
mcmcrun logpost b using temp/temp.csv, ///
  samp( (gbsgriddy , grid(-1 1 2 3 5) )     ///
        (gbsgriddy , grid(-0.2 -0.1 -0.05 0 0.1) )     ///
        (gbsgriddy , grid(0.001 0.5 1 2 5) )) ///
  burn(500) update(2000) par(alpha beta sigma) replace
import delimited temp/temp.csv, clear
replace alpha = alpha - 55*beta
gen phi = sigma/1.414214
mcmcstats alpha beta sigma phi

pause "Press q to continue"
*================================================================================

*-------------------------------
* Section 4.6
* Laplace regression
* Efficient logpost
*-------------------------------
program drop _all
use data/shoulder.dta, clear
gen y = log(pain/(10-pain) )
gen x = age - 55 
set seed 456932

*-------------------------------
* more efficient log-posterior
*-------------------------------
program logpost
    args logp b

    local alpha = `b'[1,1]
    local beta  = `b'[1,2]
    local sigma = `b'[1,3]
    local phi = `sigma'/1.414214
    tempvar u
    gen `u' = -abs(y-`alpha'-`beta'*x)/`phi' - log(`phi')
    qui summarize `u', meanonly
    scalar `logp' = r(sum) - 0.005*`alpha'*`alpha' - 50*`beta'*`beta'
end

*-------------------------------
* mcmcrun with griddy
*-------------------------------
matrix b = (1,0,1)
mcmcrun logpost b using temp/temp.csv,                      ///
  samp( (gbsgriddy , grid(-1 1 2 3 5) )                ///
        (gbsgriddy , grid(-0.2 -0.1 -0.05 0 0.1) )     ///
        (gbsgriddy , grid(0.001 0.5 1 2 5) ))          ///
  burn(500) update(2000) par(alpha beta sigma) replace
import delimited temp/temp.csv, clear
replace alpha = alpha - 55*beta
gen phi = sigma/1.414214
mcmcstats alpha beta sigma phi

pause "Press q to continue"
*================================================================================

*-------------------------------
* Section 4.6
* Laplace regression
* COMPARISON OF DIFFERENT SAMPLERS
*-------------------------------
program drop _all
use data/shoulder.dta, clear
gen y = log(pain/(10-pain) )
gen x = age - 55 
set seed 456932

*-------------------------------
* more efficient log-posterior
*-------------------------------
program logpost
    args logp b

    local alpha = `b'[1,1]
    local beta  = `b'[1,2]
    local sigma = `b'[1,3]
    local phi = `sigma'/1.414214
    tempvar u
    gen `u' = -abs(y-`alpha'-`beta'*x)/`phi' - log(`phi')
    qui summarize `u', meanonly
    scalar `logp' = r(sum) - 0.005*`alpha'*`alpha' - 50*`beta'*`beta'
end

*-------------------------------
* Metropolis-Hastings
*-------------------------------
matrix b = (1,0,1)
matrix s =(1,0.02,0.5)
mcmcrun logpost b using temp/temp.csv , ///
  samplers( (mhsnorm , sd(s) )  ///
            (mhsnorm , sd(s) ) ///
            (mhstrnc , sd(s) lb(0.001) ub(5) ) ) ///
  burnin(500) adapt update(5000)  param(alpha beta sigma)  replace dots(0) jpost
local t : display %5.2f r(updatetime)/5  
import delimited temp/temp.csv, clear
replace alpha = alpha - 55*beta
gen phi = sigma/1.414214
mcmcstats alpha beta phi
pause "Press q to continue"
*================================================================================

*-------------------------------
* Section 4.6
* Laplace regression
* COMPARISON OF DIFFERENT SAMPLERS
*-------------------------------
use data/shoulder.dta, clear
gen y = log(pain/(10-pain) )
gen x = age - 55 
set seed 456932

*-------------------------------
* Griddy 9 point
*-------------------------------
matrix b = (1,0,1)
mcmcrun logpost b using temp/temp.csv , ///
  samp( (gbsgriddy , grid(-1 0 1 1.5 2 2.5 3 4 5) )     ///
        (gbsgriddy , grid(-0.2 -0.15 -0.1 -0.075 -0.05 -0.025 0 0.05 0.1) )     ///
        (gbsgriddy , grid(0.001 0.25 0.5 0.75 1 1.5 2 3.5 5) )) ///
  burnin(500) update(5000)  param(alpha beta sigma) replace dots(0) jpost
local t : display %5.2f r(updatetime)/5  
import delimited temp/temp.csv, clear
replace alpha = alpha - 55*beta
gen phi = sigma/1.414214

mcmcstats alpha beta phi

pause "Press q to continue"
*================================================================================

*-------------------------------
* Section 4.6
* Laplace regression
* Griddy 9 histogram
*-------------------------------

use data/shoulder.dta, clear
gen y = log(pain/(10-pain) )
gen x = age - 55 
set seed 456932
matrix b = (1,0,1)
mcmcrun logpost b using temp/temp.csv , ///
  samp( (gbsgriddy , grid(-1 0 1 1.5 2 2.5 3 4 5) hist)     ///
        (gbsgriddy , grid(-0.2 -0.15 -0.1 -0.075 -0.05 -0.025 0 0.05 0.1) hist)     ///
        (gbsgriddy , grid(0.001 0.25 0.5 0.75 1 1.5 2 3.5 5) hist )) ///
  burnin(500) update(5000)  param(alpha beta sigma) replace dots(0) jpost
local t : display %5.2f r(updatetime)/5  
import delimited temp/temp.csv, clear
replace alpha = alpha - 55*beta
gen phi = sigma/1.414214

mcmcstats alpha beta phi

pause "Press q to continue"
*================================================================================

*-------------------------------
* Section 4.6
* Laplace regression
* Griddy 5 metropolis
*-------------------------------

use data/shoulder.dta, clear
gen y = log(pain/(10-pain) )
gen x = age - 55 
set seed 456932
matrix b = (1,0,1)
mcmcrun logpost b using temp/temp.csv , ///
  samp( (gbsgriddy , grid(-1 1 2 3 5) met)     ///
        (gbsgriddy , grid(-0.2 -0.1 -0.05 0 0.1) met)     ///
        (gbsgriddy , grid(0.001 0.5 1 2 5) met )) ///
  burnin(500) update(5000)  param(alpha beta sigma) replace dots(0) jpost
local t : display %5.2f r(updatetime)/5  
import delimited temp/temp.csv, clear
replace alpha = alpha - 55*beta
gen phi = sigma/1.414214

mcmcstats alpha beta phi

pause "Press q to continue"
*================================================================================

*-------------------------------
* Section 4.6
* Laplace regression
* ARMS 3
*-------------------------------
use data/shoulder.dta, clear
gen y = log(pain/(10-pain) )
gen x = age - 55 
set seed 456932
matrix b = (1,0,1)
mcmcrun logpost b using temp/temp.csv , ///
  samp( (arms , grid(1 2 3 ) )     ///
        (arms , grid(-0.1 -0.05 0 ) )     ///
        (arms , grid(0.5 1 2 ) lb(0))) ///
  burnin(500) update(5000)  param(alpha beta sigma) replace dots(0) jpost
import delimited temp/temp.csv, clear
local t : display %5.2f r(updatetime)/5  
replace alpha = alpha - 55*beta
gen phi = sigma/1.414214

mcmcstats alpha beta phi

pause "Press q to continue"
*================================================================================

*-------------------------------
* Section 4.6
* Laplace regression
* ARMS 9
*-------------------------------

use data/shoulder.dta, clear
gen y = log(pain/(10-pain) )
gen x = age - 55 

matrix b = (1,0,1)
mcmcrun logpost b using temp/temp.csv , ///
  samp( (arms , grid(-1 0 1 1.5 2 2.5 3 4 5) )     ///
        (arms , grid(-0.2 -0.15 -0.1 -0.075 -0.05 -0.025 0 0.05 0.1) )     ///
        (arms , grid(0.001 0.25 0.5 0.75 1 1.5 2 3.5 5) lb(0) )) ///
  burnin(500) update(5000)  param(alpha beta sigma) replace dots(0) jpost
import delimited temp/temp.csv, clear
local t : display %5.2f r(updatetime)/5  
replace alpha = alpha - 55*beta
gen phi = sigma/1.414214

mcmcstats alpha beta phi

pause "Press q to continue"
*================================================================================

*-------------------------------
* Section 4.6
* Laplace regression
* SLICE small steps
*-------------------------------

use data/shoulder.dta, clear
gen y = log(pain/(10-pain) )
gen x = age - 55 
set seed 456932
matrix b = (1,0,1)
mcmcrun logpost b using temp/temp.csv , ///
  samplers( (slice , lb(-5)   ub(5)  step(1.5) )  ///
            (slice , lb(-0.2)   ub(0)  step(0.025) ) ///
            (slice , lb(0)   ub(5)  step(0.3) ) ) ///
  burnin(500) update(5000)  param(alpha beta sigma) replace dots(0) jpost
local t : display %5.2f r(updatetime)/5  
import delimited temp/temp.csv, clear
replace alpha = alpha - 55*beta
gen phi = sigma/1.414214

mcmcstats alpha beta phi

pause "Press q to continue"
*================================================================================

*-------------------------------
* Section 4.6
* Laplace regression
* SLICE
*-------------------------------
use data/shoulder.dta, clear
gen y = log(pain/(10-pain) )
gen x = age - 55 
set seed 456932
matrix b = (1,0,1)
mcmcrun logpost b using temp/temp.csv , ///
  samplers( (slice , lb(-5)   ub(5)  step(3) )  ///
            (slice , lb(-0.2)   ub(0)  step(0.05) ) ///
            (slice , lb(0)   ub(5)  step(0.6) ) ) ///
  burnin(500) update(5000)  param(alpha beta sigma) replace dots(0) jpost
local t : display %5.2f r(updatetime)/5  
import delimited temp/temp.csv, clear
replace alpha = alpha - 55*beta
gen phi = sigma/1.414214

mcmcstats alpha beta phi

pause "Press q to continue"
*================================================================================

*-------------------------------
* Section 4.6
* Laplace regression
* SLICE big steps
*-------------------------------

use data/shoulder.dta, clear
gen y = log(pain/(10-pain) )
gen x = age - 55 
set seed 456932
matrix b = (1,0,1)
mcmcrun logpost b using temp/temp.csv , ///
  samplers( (slice , lb(-5)   ub(5)  step(6) )  ///
            (slice , lb(-0.2)   ub(0)  step(0.1) ) ///
            (slice , lb(0)   ub(5)  step(1.2) ) ) ///
  burnin(500) update(5000)  param(alpha beta sigma) replace dots(0) jpost
local t : display %5.2f r(updatetime)/5  
import delimited temp/temp.csv, clear
replace alpha = alpha - 55*beta
gen phi = sigma/1.414214

mcmcstats alpha beta phi

*================================================================================
