*------------------------------
* Code for Chapter 6
*------------------------------
program drop _all
clear
set scheme sj
set more off
pause off
local tupdates 1000 // query-replace updates(20000) with updates(`tupdates')
local contour 250   // query-replace 1/5000 with 1/`contour'
local whatfile chapter6t.do // query-replace chapter6.do `whatfile'

*-------------------------------
* Section 6.2
* Implicit prior on theta
* figure 6.1
*-------------------------------
set obs 10000
gen alpha1 = rnormal(0,1)
gen beta2  = rnormal(0,0.5)
gen beta3  = rnormal(0,0.5)
gen delta2 = rgamma(5,0.2)
gen delta3 = rgamma(5,0.2)
gen delta4 = rgamma(5,0.2)
gen alpha2 = alpha1 + delta2
gen alpha3 = alpha2 + delta3
gen alpha4 = alpha3 + delta4
gen beta1 = 0
forvalues k=1/3 {
  gen cp0 = 0
  forvalues j=1/4 {
     local h = `j' - 1
     gen cp`j' = invlogit(alpha`j'+beta`k')
     gen p`j'_`k' = cp`j' - cp`h'
  }
  gen p5_`k' = 1 - cp4
  drop cp*
}
gen theta12 = p1_1*(p2_2+p3_2+p4_2+p5_2)+ p2_1*(p3_2+p4_2+p5_2) ///
   + p3_1*(p4_2+p5_2) + p4_1*p5_2 + 0.5*(p1_1*p1_2 +   ///
     p2_1*p2_2 + p3_1*p3_2 + p4_1*p4_2 +p5_1*p5_2)
gen theta13 = p1_1*(p2_3+p3_3+p4_3+p5_3)+ p2_1*(p3_3+p4_3+p5_3) ///
   + p3_1*(p4_3+p5_3) + p4_1*p5_3 + 0.5*(p1_1*p1_3 +   ///
     p2_1*p2_3 + p3_1*p3_3 + p4_1*p4_3 +p5_1*p5_3)
gen theta23 = p1_2*(p2_3+p3_3+p4_3+p5_3)+ p2_2*(p3_3+p4_3+p5_3) ///
   + p3_2*(p4_3+p5_3) + p4_2*p5_3 + 0.5*(p1_2*p1_3 +   ///
     p2_2*p2_3 + p3_2*p3_3 + p4_2*p4_3 +p5_2*p5_3)

summarize theta*    
* FIGURE 6.1  
mcmcdensity theta12, gopt(yscale(off) xlabel(0.2(0.1)0.8) xtitle(Theta))
graph export eps\chap6_theta_prior.eps, replace

pause "Press q to continue"
*=============================================================================
*-------------------------------
* Section 6.2
* fit ordinal regression
*-------------------------------
program drop _all
*-------------------------------
* Data Matrix
*-------------------------------
matrix C = (50,9,4,3,0\24,13,14,2,0\31,14,8,2,0)

*-------------------------------
* program for log-posterior
*-------------------------------
program logpost
   args logp b

   local alpha1 = `b'[1,1]
   local beta2  = `b'[1,2]
   local beta3  = `b'[1,3]
   local delta2 = `b'[1,4]
   local delta3 = `b'[1,5]
   local delta4 = `b'[1,6]
   local alpha2 = `alpha1' + `delta2'
   local alpha3 = `alpha2' + `delta3'
   local alpha4 = `alpha3' + `delta4'
   local beta1 = 0
   local cp0 = 0
   forvalues k=1/3 {
      forvalues j=1/4 {
         local h = `j' - 1
         local cp`j' = invlogit(`alpha`j''+`beta`k'')
         local p`j'_`k' = `cp`j'' - `cp`h''
      }
      local p5_`k' = 1 - `cp4'
   }
   scalar `logp' = 0
   forvalues k=1/3 {
      forvalues j=1/5 {
         scalar `logp'= `logp' + C[`k',`j']*log(`p`j'_`k'')
      }
   }
   logdensity normal `logp'  `alpha1' 0 1
   logdensity normal `logp'  `beta2'  0 0.5
   logdensity normal `logp'  `beta3'  0 0.5
   logdensity gamma  `logp'  `delta2' 5 0.2
   logdensity gamma  `logp'  `delta3' 5 0.2
   logdensity gamma  `logp'  `delta4' 5 0.2
end

*-------------------------------
* fit using mcmcrun
*-------------------------------
matrix b = (0,0,0,1,1,1)
matrix sd = J(1,6,0.5)
mcmcrun logpost b using temp/temp.csv, samp( 3(mhsnorm, sd(sd)) 3(mhslogn, sd(sd)) ) ///
  burnin(1000) adapt updates(20000) thin(4) ///
      replace par( alpha1 beta2 beta3 delta2-delta4) ///
      dots(0) jpost
import delimited temp/temp.csv, clear
gen alpha2 = alpha1 + delta2
gen alpha3 = alpha2 + delta3
gen alpha4 = alpha3 + delta4
mcmcstats alpha* beta* delta*
save temp/ordinal_mcmc.dta, replace

pause "Press q to continue"
*=============================================================================
*-------------------------------
* Section 6.3
* validation
*-------------------------------
clear
program drop _all
*-------------------------------
* program for log-posterior
* replace missing logp with large negative value
*-------------------------------
program logpost
   args logp b

   local alpha1 = `b'[1,1]
   local beta2  = `b'[1,2]
   local beta3  = `b'[1,3]
   local delta2 = `b'[1,4]
   local delta3 = `b'[1,5]
   local delta4 = `b'[1,6]
   local alpha2 = `alpha1' + `delta2'
   local alpha3 = `alpha2' + `delta3'
   local alpha4 = `alpha3' + `delta4'
   local beta1 = 0
   local cp0 = 0
   forvalues k=1/3 {
      forvalues j=1/4 {
         local h = `j' - 1
         local cp`j' = invlogit(`alpha`j''+`beta`k'')
         local p`j'_`k' = `cp`j'' - `cp`h''
      }
      local p5_`k' = 1 - `cp4'
   }
   scalar `logp' = 0
   forvalues k=1/3 {
      forvalues j=1/5 {
         scalar `logp'= `logp' + C[`k',`j']*log(`p`j'_`k'')
      }
   }
   logdensity normal `logp'  `alpha1' 0 1
   logdensity normal `logp'  `beta2'  0 0.5
   logdensity normal `logp'  `beta3'  0 0.5
   logdensity gamma  `logp'  `delta2' 5 0.2
   logdensity gamma  `logp'  `delta3' 5 0.2
   logdensity gamma  `logp'  `delta4' 5 0.2
   if missing(`logp')  scalar `logp' = -9E10
end

*-------------------------------
* generate random data matrix
* to create tex file
*-------------------------------
qui set obs 174
gen trt = 1 + (_n>66) + (_n>119)
gen u = runiform()
gen appetite = 1
local alpha1 = rnormal(0,1)
local beta2  = rnormal(0,0.5)
local beta3  = rnormal(0,0.5)
local delta2 = rgamma(5,0.2)
local delta3 = rgamma(5,0.2)
local delta4 = rgamma(5,0.2)
local alpha2 = `alpha1' + `delta2'
local alpha3 = `alpha2' + `delta3'
local alpha4 = `alpha3' + `delta4'
local beta1 = 0
forvalues k=1/3 {
   forvalues j=1/4 {
      local cp`j' = invlogit(`alpha`j''+`beta`k'')
      qui replace appetite = appetite + 1 if u > `cp`j'' & trt == `k' 
   }
}
matrix C = J(3,5,0)
forvalues i=1/3 {
   forvalues j=1/5 {
      qui count if trt == `i' & appetite == `j'
      matrix C[`i',`j'] = r(N)
   }
}

pause "Press q to continue"
*=============================================================================

clear
program drop _all
*-------------------------------
* tex file needs editting to
* remove code already given
*-------------------------------
program logpost
   args logp b

   local alpha1 = `b'[1,1]
   local beta2  = `b'[1,2]
   local beta3  = `b'[1,3]
   local delta2 = `b'[1,4]
   local delta3 = `b'[1,5]
   local delta4 = `b'[1,6]
   local alpha2 = `alpha1' + `delta2'
   local alpha3 = `alpha2' + `delta3'
   local alpha4 = `alpha3' + `delta4'
   local beta1 = 0
   local cp0 = 0
   forvalues k=1/3 {
      forvalues j=1/4 {
         local h = `j' - 1
         local cp`j' = invlogit(`alpha`j''+`beta`k'')
         local p`j'_`k' = `cp`j'' - `cp`h''
      }
      local p5_`k' = 1 - `cp4'
   }
   scalar `logp' = 0
   forvalues k=1/3 {
      forvalues j=1/5 {
         scalar `logp'= `logp' + C[`k',`j']*log(`p`j'_`k'')
      }
   }
   logdensity normal `logp'  `alpha1' 0 1
   logdensity normal `logp'  `beta2'  0 0.5
   logdensity normal `logp'  `beta3'  0 0.5
   logdensity gamma  `logp'  `delta2' 5 0.2
   logdensity gamma  `logp'  `delta3' 5 0.2
   logdensity gamma  `logp'  `delta4' 5 0.2
   if missing(`logp')  scalar `logp' = -9E10
end

tempname rf
postfile `rf' alpha1 q1 m1 sd1 se1 beta2  q2 m2 sd2 se2 ///
  beta3  q3 m3 sd3 se3 delta2 q4 m4 sd4 se4 ///
  delta3 q5 m5 sd5 se5 delta4 q6 m6 sd6 se6 ///
  using temp/ordinal_valid.dta,replace
set seed 639125
forvalues rep=1/20 {
   qui drop _all
   qui set obs 174
   gen trt = 1 + (_n>66) + (_n>119)
   gen u = runiform()
   gen appetite = 1
   local alpha1 = rnormal(0,1)
   local beta2  = rnormal(0,0.5)
   local beta3  = rnormal(0,0.5)
   local delta2 = rgamma(5,0.2)
   local delta3 = rgamma(5,0.2)
   local delta4 = rgamma(5,0.2)        
   local alpha2 = `alpha1' + `delta2'
   local alpha3 = `alpha2' + `delta3'
   local alpha4 = `alpha3' + `delta4'
   local beta1 = 0
   forvalues k=1/3 {
      forvalues j=1/4 {
         local cp`j' = invlogit(`alpha`j''+`beta`k'')
         qui replace appetite = appetite + 1 if u > `cp`j'' & trt == `k' 
      }
   }
   matrix C = J(3,5,0)
   forvalues i=1/3 {
      forvalues j=1/5 {
         qui count if trt == `i' & appetite == `j'
         matrix C[`i',`j'] = r(N)
      }
   }
   matrix b = (`alpha1',`beta2',`beta3',`delta2',`delta3',`delta4')
   matrix sd = J(1,6,0.5)
   mcmcrun logpost b using temp/temp.csv,                  ///
      samp( 3(mhsnorm, sd(sd)) 3(mhslogn, sd(sd)) ) ///
      burnin(1000) adapt updates(20000) thin(4)       ///
      replace par( alpha1 beta2 beta3 delta2-delta4) dots(0) jpost
   import delimited temp/temp.csv, clear
   local n = _N
   local postlist ""
   foreach v of varlist a* b* d* {
      qui count if `v' < ``v''
      local q`v' = (r(N)+0.5)/(`n'+1)
      qui mcmcstats `v'
      local postlist "`postlist' (``v'') (`q`v'')"
      local postlist "`postlist' (`r(mn1)') (`r(sd1)') (`r(se1)')"
   }
   post `rf' `postlist'
}
postclose `rf'

use temp/ordinal_valid.dta, clear
local i = 0
foreach v of varlist q* {
   local ++i
   gen z`i' = invnormal(q`i')
   gen c2`i' = z`i'*z`i'
   qui summarize c2`i'
   display %6s "`v'" %8.3f `r(sum)' %8.4f chi2tail(_N,`r(sum)')
}

pause "Press q to continue"
*=============================================================================

program drop _all
*--------------------------
* Data matrix
*--------------------------
matrix C = (50,9,4,3,0\24,13,14,2,0\31,14,8,2,0)
*--------------------------
* log-posterior
*--------------------------
program logpost
   args logp b

   local alpha1 = `b'[1,1]
   local beta2  = `b'[1,2]
   local beta3  = `b'[1,3]
   local delta2 = `b'[1,4]
   local delta3 = `b'[1,5]
   local delta4 = `b'[1,6]
   local alpha2 = `alpha1' + `delta2'
   local alpha3 = `alpha2' + `delta3'
   local alpha4 = `alpha3' + `delta4'

   local beta1 = 0
   local cp0 = 0
   forvalues k=1/3 {
      forvalues j=1/4 {
         local h = `j' - 1
         local cp`j' = invlogit(`alpha`j''+`beta`k'')
         local p`j'_`k' = `cp`j'' - `cp`h''
      }
      local p5_`k' = 1 - `cp4'
   }
   scalar `logp' = 0
   forvalues k=1/3 {
      forvalues j=1/5 {
         scalar `logp'= `logp' + C[`k',`j']*log(`p`j'_`k'')
      }
   }
   logdensity normal `logp'  `alpha1' 0 1
   logdensity normal `logp'  `beta2'  0 0.5
   logdensity normal `logp'  `beta3'  0 0.5
   logdensity gamma  `logp'  `delta2' 5 0.2
   logdensity gamma  `logp'  `delta3' 5 0.2
   logdensity gamma  `logp'  `delta4' 5 0.2
end
*--------------------------
* fit using mcmcrun
*--------------------------
matrix b = (0,0,0,1,1,1)
matrix sd = J(1,6,0.5)
mcmcrun logpost b using temp/temp.csv, ///
   samp( 3(mhsnorm, sd(sd)) 3(mhslogn, sd(sd)) ) ///
   burnin(1000) adapt updates(20000) thin(4) ///
   replace par( alpha1 beta2 beta3 delta2-delta4) ///
   dots(0) jpost
import delimited temp/temp.csv, clear
gen alpha2 = alpha1 + delta2
gen alpha3 = alpha2 + delta3
gen alpha4 = alpha3 + delta4
mcmcstats alpha* beta* delta*
save temp/ordinal_mcmc.dta, replace

pause "Press q to continue"
*=============================================================================
*-------------------------------
* Section 6.3
* validation
*-------------------------------
program drop _all
program logpost
   args logp b

   local alpha1 = `b'[1,1]
   local beta2  = `b'[1,2]
   local beta3  = `b'[1,3]
   local delta2 = `b'[1,4]
   local delta3 = `b'[1,5]
   local delta4 = `b'[1,6]
   local alpha2 = `alpha1' + `delta2'
   local alpha3 = `alpha2' + `delta3'
   local alpha4 = `alpha3' + `delta4'

   local beta1 = 0
   local cp0 = 0
   forvalues k=1/3 {
      forvalues j=1/4 {
         local h = `j' - 1
         local cp`j' = invlogit(`alpha`j''+`beta`k'')
         local p`j'_`k' = `cp`j'' - `cp`h''
      }
      local p5_`k' = 1 - `cp4'
   }
   scalar `logp' = 0
   forvalues k=1/3 {
      forvalues j=1/5 {
         scalar `logp'= `logp' + C[`k',`j']*log(`p`j'_`k'')
      }
   }
   logdensity normal `logp'  `alpha1' 0 1
   logdensity normal `logp'  `beta2'  0 0.5
   logdensity normal `logp'  `beta3'  0 0.5
   logdensity gamma  `logp'  `delta2' 5 0.2
   logdensity gamma  `logp'  `delta3' 5 0.2
   logdensity gamma  `logp'  `delta4' 5 0.2
   if missing(`logp')  scalar `logp' = -9E10
end

clear
set seed 639125
*--------------------------------
* File to store results
*--------------------------------
tempname rf
postfile `rf' alpha1 q1 m1 sd1 se1 beta2  q2 m2 sd2 se2 ///
   beta3  q3 m3 sd3 se3 delta2 q4 m4 sd4 se4 ///
   delta3 q5 m5 sd5 se5 delta4 q6 m6 sd6 se6 ///
   using temp/ordinal_valid.dta,replace

*--------------------------------
* Run 20 times
*--------------------------------
forvalues rep=1/20 {
   qui drop _all
*--------------------------------
* Simulate new data from prior
*--------------------------------
   qui set obs 174
   gen trt = 1 + (_n>66) + (_n>119)
   gen u = runiform()
   gen appetite = 1

   local alpha1 = rnormal(0,1)
   local beta2  = rnormal(0,0.5)
   local beta3  = rnormal(0,0.5)
   local delta2 = rgamma(5,0.2)
   local delta3 = rgamma(5,0.2)
   local delta4 = rgamma(5,0.2)
        
   local alpha2 = `alpha1' + `delta2'
   local alpha3 = `alpha2' + `delta3'
   local alpha4 = `alpha3' + `delta4'

   local beta1 = 0
   forvalues k=1/3 {
      forvalues j=1/4 {
         local cp`j' = invlogit(`alpha`j''+`beta`k'')
         qui replace appetite = appetite + 1 if u > `cp`j'' & trt == `k' 
      }
   }
   matrix C = J(3,5,0)
   forvalues i=1/3 {
      forvalues j=1/5 {
         qui count if trt == `i' & appetite == `j'
         matrix C[`i',`j'] = r(N)
      }
   }
*--------------------------------
* fit with mcmcrun
*--------------------------------
   matrix b = (`alpha1',`beta2',`beta3',`delta2',`delta3',`delta4')
   matrix sd = J(1,6,0.5)
   mcmcrun logpost b using temp/temp.csv, ///
      samp( 3(mhsnorm, sd(sd)) 3(mhslogn, sd(sd)) ) ///
      burnin(1000) adapt updates(20000) thin(4) ///
      replace par( alpha1 beta2 beta3 delta2-delta4) ///
      dots(0) jpost
*--------------------------------
* Save summary statistics
*--------------------------------	  
   import delimited temp/temp.csv, clear
   local n = _N
   local postlist ""
   foreach v of varlist a* b* d* {
      qui count if `v' < ``v''
      local q`v' = (r(N)+0.5)/(`n'+1)
      qui mcmcstats `v'
      local postlist "`postlist' (``v'') (`q`v'')"
      local postlist "`postlist' (`r(mn1)') (`r(sd1)') (`r(se1)')"
   }
   post `rf' `postlist'
}
postclose `rf'
*--------------------------------
* Calculate p-values
*--------------------------------
use temp/ordinal_valid.dta, clear
local i = 0
foreach v of varlist q* {
   local ++i
   gen z`i' = invnormal(q`i')
   gen c2`i' = z`i'*z`i'
   qui summarize c2`i'
   display %6s "`v'" %8.3f `r(sum)' %8.4f chi2tail(_N,`r(sum)')
}

pause "Press q to continue"
*=============================================================================
*-------------------------------
* Section 6.3
* validation - with error
*-------------------------------
program drop _all
set seed 639125

program logpost
   args logp b

   local alpha1 = `b'[1,1]
   local beta2  = `b'[1,2]
   local beta3  = `b'[1,3]
   local delta2 = `b'[1,4]
   local delta3 = `b'[1,5]
   local delta4 = `b'[1,6]
   local alpha2 = `alpha1' + `delta2'
   local alpha3 = `alpha2' + `delta3'
   local alpha4 = `alpha3' + `delta4'

   local beta1 = 0
   local cp0 = 0
   forvalues k=1/3 {
      forvalues j=1/4 {
         local h = `j' - 1
* ERROR - in place of +
         local cp`j' = invlogit(`alpha`j''-`beta`k'')
         local p`j'_`k' = `cp`j'' - `cp`h''
      }
      local p5_`k' = 1 - `cp4'
   }
   scalar `logp' = 0
   forvalues k=1/3 {
      forvalues j=1/5 {
         scalar `logp'= `logp' + C[`k',`j']*log(`p`j'_`k'')
      }
   }
   logdensity normal `logp'  `alpha1' 0 1
   logdensity normal `logp'  `beta2'  0 0.5
   logdensity normal `logp'  `beta3'  0 0.5
   logdensity gamma  `logp'  `delta2' 5 0.2
   logdensity gamma  `logp'  `delta3' 5 0.2
   logdensity gamma  `logp'  `delta4' 5 0.2
   if missing(`logp')  scalar `logp' = -9E10
end

clear
*--------------------------------
* File to store results
*--------------------------------
tempname rf
postfile `rf' alpha1 q1 m1 sd1 se1 beta2  q2 m2 sd2 se2 ///
  beta3  q3 m3 sd3 se3 delta2 q4 m4 sd4 se4 ///
  delta3 q5 m5 sd5 se5 delta4 q6 m6 sd6 se6 ///
  using temp/ordinal_error.dta,replace

*--------------------------------
* Run 20 times
*--------------------------------
forvalues rep=1/20 {
   qui drop _all
*--------------------------------
* Simulate new data from prior
*--------------------------------
   qui set obs 174
   gen trt = 1 + (_n>66) + (_n>119)
   gen u = runiform()
   gen appetite = 1

   local alpha1 = rnormal(0,1)
   local beta2  = rnormal(0,0.5)
   local beta3  = rnormal(0,0.5)
   local delta2 = rgamma(5,0.2)
   local delta3 = rgamma(5,0.2)
   local delta4 = rgamma(5,0.2)
        
   local alpha2 = `alpha1' + `delta2'
   local alpha3 = `alpha2' + `delta3'
   local alpha4 = `alpha3' + `delta4'

   local beta1 = 0
   forvalues k=1/3 {
      forvalues j=1/4 {
         local cp`j' = invlogit(`alpha`j''+`beta`k'')
         qui replace appetite = appetite + 1 if u > `cp`j'' & trt == `k' 
      }
   }
   matrix C = J(3,5,0)
   forvalues i=1/3 {
      forvalues j=1/5 {
         qui count if trt == `i' & appetite == `j'
         matrix C[`i',`j'] = r(N)
      }
   }
*--------------------------------
* fit with mcmcrun
*--------------------------------
   matrix b = (`alpha1',`beta2',`beta3',`delta2',`delta3',`delta4')
   matrix sd = J(1,6,0.5)
   mcmcrun logpost b using temp/temp.csv, ///
      samp( 3(mhsnorm, sd(sd)) 3(mhslogn, sd(sd)) ) ///
      burnin(1000) adapt updates(20000) thin(4) ///
      replace par( alpha1 beta2 beta3 delta2-delta4) ///
      dots(0) jpost
*--------------------------------
* Save summary statistics
*--------------------------------	  
   import delimited temp/temp.csv, clear
   local n = _N
   local postlist ""
   foreach v of varlist a* b* d* {
      qui count if `v' < ``v''
      local q`v' = (r(N)+0.5)/(`n'+1)
      qui mcmcstats `v'
      local postlist "`postlist' (``v'') (`q`v'')"
      local postlist "`postlist' (`r(mn1)') (`r(sd1)') (`r(se1)')"
   }
   post `rf' `postlist'
}
postclose `rf'
*--------------------------------
* Calculate p-values
*--------------------------------
use temp/ordinal_error.dta, clear
local n = 5000
local i = 0
foreach v of varlist q* {
   local ++i
   qui replace q`i' = (`n'*q`i'+0.5)/(`n'+1)
   gen z`i' = invnormal(q`i')
   gen c2`i' = z`i'*z`i'
   qui summarize c2`i'
   display %6s "`v'" %8.3f `r(sum)' %8.4f chi2tail(_N,`r(sum)')
}


pause "Press q to continue"
*=============================================================================
*-------------------------------
* Section 6.4
* summary of ordinal fit
* numbers vary slightly according to the randomness 
* of the simulations
*-------------------------------
use temp/ordinal_mcmc.dta, clear

mcmcstats alpha1 be* de*, corr

pause "Press q to continue"
*=============================================================================
*-------------------------------
* Section 6.4
* figure 6.2
*-------------------------------

*-------------------------------
* diagram to show Credible Interval
*-------------------------------
clear
range x 0 15 2000
gen y = normalden(log(x),1,0.5)
gen x1 = .
gen y1 = .
replace x1 = 1.5 in 1
replace y1 = 0 in 1
replace x1 = 1.5 in 2
replace y1 = normalden(log(1.5),1,0.5) in 2
gen x2 = .
gen y2 = .
replace x2 = 8 in 1
replace y2 = 0 in 1
replace x2 = 8 in 2
replace y2 = normalden(log(8),1,0.5) in 2
twoway (line y x) (line y1 x1, lpat(solid)) ///
   (area y x if x < 1.5, fcol(black) fin(10)) ///
   (line y2 x2, lpat(solid)) (area y x if x >8, fin(10) fcol(black) ) ///
   , legend(off) yline(0,lpat(solid)) ytitle(" ") xtitle("theta") ///
   ylabel(none)
graph export eps\credible.eps, replace
pause "Press q to continue"
*=============================================================================
*-------------------------------
* diagram to show HPD   
*-------------------------------
clear
range x 0 15 2000
gen y = normalden(log(x),1,0.5)
gen x1 = .
gen y1 = .
replace x1 = 1.11 in 1
replace y1 = 0 in 1
replace x1 = 1.11 in 2
replace y1 = normalden(log(1.11),1,0.5) in 2
gen x2 = .
gen y2 = .
replace x2 = 7 in 1
replace y2 = 0 in 1
replace x2 = 7 in 2
replace y2 = normalden(log(7),1,0.5) in 2
twoway (line y x) (line y1 x1, lpat(solid)) (area y x if x < 1.11, fcol(black) fin(10)) ///
   (line y2 x2, lpat(solid)) (area y x if x >7, fin(10) fcol(black) ) ///
   , legend(off) yline(0,lpat(solid)) ytitle(" ") xtitle("theta") ///
   ylabel(none)
* FIGURE 6.3  
graph export eps\hpd.eps, replace
pause "Press q to continue"
*=============================================================================
*-------------------------------
* Section 6.4
* summary of ordinal fit
* figure 6.3
*-------------------------------
use temp/ordinal_mcmc.dta, clear
mcmcstats alpha1 b* d*, save(ordinal_stats.dta,replace)
use ordinal_stats.dta, clear
gen t = _n
twoway (scatter mean t) (rcap lb ub t), leg(off) yline(0) xtitle("") ///
   xlabel(1 "alpha" 2 "beta2" 3 "beta3" 4 "delta2" 5 "delta3" 6 "delta4")
* FIGURE 6.3  
graph export eps\chap6_cris.eps, replace

pause "Press q to continue"
*=============================================================================
*-------------------------------
* Section 6.4
* summarize theta
*-------------------------------
use temp/ordinal_mcmc.dta, clear

gen beta1 = 0
forvalues k=1/3 {
   gen cp0 = 0
   forvalues j=1/4 {
      local h = `j' - 1
      gen cp`j' = invlogit(alpha`j'+beta`k')
      gen p`j'_`k' = cp`j' - cp`h'
   }
   gen p5_`k' = 1 - cp4
   drop cp*
}
gen theta12 = p1_1*(p2_2+p3_2+p4_2+p5_2)+ p2_1*(p3_2+p4_2+p5_2) ///
   + p3_1*(p4_2+p5_2) + p4_1*p5_2 + 0.5*(p1_1*p1_2 + ///
   p2_1*p2_2 + p3_1*p3_2 + p4_1*p4_2 +p5_1*p5_2)
gen theta13 = p1_1*(p2_3+p3_3+p4_3+p5_3)+ p2_1*(p3_3+p4_3+p5_3) ///
   + p3_1*(p4_3+p5_3) + p4_1*p5_3 + 0.5*(p1_1*p1_3 + ///
   p2_1*p2_3 + p3_1*p3_3 + p4_1*p4_3 +p5_1*p5_3)
gen theta23 = p1_2*(p2_3+p3_3+p4_3+p5_3)+ p2_2*(p3_3+p4_3+p5_3) ///
   + p3_2*(p4_3+p5_3) + p4_2*p5_3 + 0.5*(p1_2*p1_3 + ///
   p2_2*p2_3 + p3_2*p3_3 + p4_2*p4_3 +p5_2*p5_3)

mcmcstats theta*
gen dtheta = theta12 - theta13
mcmcstats dtheta

pause "Press q to continue"
*=============================================================================
*-------------------------------
* Section 6.4
* refit with wider priors
* post theta
*-------------------------------
program drop _all
*-----------------------------
* Data Matrix
*-----------------------------
matrix C = (50,9,4,3,0\24,13,14,2,0\31,14,8,2,0)
*-----------------------------
* log-posterior with wide prior
*-----------------------------
program logpost
   args logp b

   local alpha1 = `b'[1,1]
   local beta2  = `b'[1,2]
   local beta3  = `b'[1,3]
   local delta2 = `b'[1,4]
   local delta3 = `b'[1,5]
   local delta4 = `b'[1,6]
   local alpha2 = `alpha1' + `delta2'
   local alpha3 = `alpha2' + `delta3'
   local alpha4 = `alpha3' + `delta4'

   local beta1 = 0
   local cp0 = 0
   forvalues k=1/3 {
      forvalues j=1/4 {
         local h = `j' - 1
         local cp`j' = invlogit(`alpha`j''+`beta`k'')
         local p`j'_`k' = `cp`j'' - `cp`h''
      }
      local p5_`k' = 1 - `cp4'
   }
   scalar `logp' = 0
   forvalues k=1/3 {
      forvalues j=1/5 {
         scalar `logp'= `logp' + C[`k',`j']*log(`p`j'_`k'')
      }
   }
   logdensity normal `logp'  `alpha1' 0 3
   logdensity normal `logp'  `beta2'  0 3
   logdensity normal `logp'  `beta3'  0 3
   logdensity gamma  `logp'  `delta2' 5 0.2
   logdensity gamma  `logp'  `delta3' 5 0.2
   logdensity gamma  `logp'  `delta4' 5 0.2
   if missing(`logp') scalar `logp' = -9E20
end
*-----------------------------
* fit with mcmcrun
*-----------------------------
matrix b = (0,0,0,1,1,1)
matrix sd = J(1,6,0.5)
mcmcrun logpost b using temp/temp.csv, ///
   samp( 3(mhsnorm, sd(sd)) 3(mhslogn, sd(sd)) ) ///
   burnin(1000) adapt updates(20000) thin(4) ///
   replace par( alpha1 beta2 beta3 delta2-delta4) ///
   dots(0) jpost
import delimited temp/temp.csv, clear
*------------------------------------
* Fitted probabilities
*------------------------------------
gen alpha2 = alpha1 + delta2
gen alpha3 = alpha2 + delta3
gen alpha4 = alpha3 + delta4
gen beta1 = 0
forvalues k=1/3 {
   gen cp0 = 0
   forvalues j=1/4 {
      local h = `j' - 1
      gen cp`j' = invlogit(alpha`j'+beta`k')
      gen p`j'_`k' = cp`j' - cp`h'
   }
   gen p5_`k' = 1 - cp4
   drop cp*
}
*------------------------------------
* Fitted theta's
*------------------------------------
gen theta12 = p1_1*(p2_2+p3_2+p4_2+p5_2)+ p2_1*(p3_2+p4_2+p5_2) ///
   + p3_1*(p4_2+p5_2) + p4_1*p5_2 + 0.5*(p1_1*p1_2 + ///
   p2_1*p2_2 + p3_1*p3_2 + p4_1*p4_2 +p5_1*p5_2)
gen theta13 = p1_1*(p2_3+p3_3+p4_3+p5_3)+ p2_1*(p3_3+p4_3+p5_3) ///
   + p3_1*(p4_3+p5_3) + p4_1*p5_3 + 0.5*(p1_1*p1_3 + ///
   p2_1*p2_3 + p3_1*p3_3 + p4_1*p4_3 +p5_1*p5_3)
gen theta23 = p1_2*(p2_3+p3_3+p4_3+p5_3)+ p2_2*(p3_3+p4_3+p5_3) ///
   + p3_2*(p4_3+p5_3) + p4_2*p5_3 + 0.5*(p1_2*p1_3 + ///
   p2_2*p2_3 + p3_2*p3_3 + p4_2*p4_3 +p5_2*p5_3)
mcmcstats theta*

pause "Press q to continue"
*=============================================================================
*-------------------------------
* Section 6.5
* densities
* figure 6.4
*-------------------------------
use temp/ordinal_mcmc.dta, clear
* FIGURE 6.4  
mcmcdensity alpha1 de* beta2 beta3
graph export eps\chap6_density.eps, replace

pause "Press q to continue"
*=============================================================================
*-------------------------------
* Section 6.5
* prior & posterior for delta4
* figure 6.5
*-------------------------------
use temp/ordinal_mcmc.dta, clear
mcmcdensity delta4, addplot(function y=gammaden(5,0.2,0,x), range(0 4))
* FIGURE 6.5  
graph export eps\chap6_delta4.eps, replace

pause "Press q to continue"
*=============================================================================
*-------------------------------
* Section 6.5
* matrix plot
* figure 6.6
*-------------------------------
use temp/ordinal_mcmc.dta, clear
* FIGURE 6.6  
graph matrix alpha1 de* beta2 beta3
graph export eps\chap6_matrix.eps, replace

pause "Press q to continue"
*=============================================================================
*-------------------------------
* Section 6.5
* contour plot
* figure 6.7
*-------------------------------
use temp/ordinal_mcmc.dta, clear
gen z = 0
forvalues i=1/5000 {
   local m1 = beta2[`i']
   local m2 = beta3[`i']
   gen tmp = normalden(beta2,`m1',0.1)*normalden(beta3,`m2',0.1)
   qui summarize tmp
   qui replace z = r(sum) in `i'
   drop tmp
}
sort beta2 
twoway contour z beta3 beta2 if mod(_n,50) == 0, level(10) clegend(off) ///
   xscale(range(-1.4 0.2)) xlabel(-1.4(0.2)0.2)  
* FIGURE 6.6  
graph export eps\chap6_contour.eps, replace

pause "Press q to continue"
*=============================================================================
   
