*------------------------------
* Code for Chapter 8
* working directory must include the data file: data\dudongs.txt data\jaw.dta
*------------------------------
clear
set more off
set scheme sj
program drop _all
pause off

*-------------------------------
* Section 8.2
* Simple model
*-------------------------------
tomodel chapter8.do temp/model.txt V1
/*
model {
   for (i in 1:21) {
      logit(p[i]) <- a0 + a1*x1[i] + a2*x2[1] + a3*x12[i] + u[i]
      y[i] ~ dbin(p[i],n[i])
      u[i] ~ dnorm(0,tau)
   }
   a0 ~ dnorm(0,0.25)
   a1 ~ dnorm(0,0.25)
   a2 ~ dnorm(0,0.25)
   a3 ~ dnorm(0,0.25)
   tau ~ dgamma(2,0.5)
}   
*/

use data\broomrape.dta, clear
summarize seed 
local m1 = r(mean)
gen x1 = seed - `m1'
summarize root 
local m2 = r(mean)
gen x2 = root - `m2'
gen x12 = root*seed
summarize x12 
local m12 = r(mean)
replace x12 = x12 - `m12'
wbslist (vector y n x1 x2 x12, format(%5.2f)) using temp/data.txt, replace 
wbslist (a0=0,a1=0,a2=0,a3=0,tau=1) using temp/init.txt,  replace

wbsscript using temp/script.txt, model(model.txt) data(data.txt) ///
    inits(init.txt)  log(log.txt)   ///
    coda(out) set(a0 a1 a2 a3 tau ) ///
    burn(1000) update(10000) replace openbugs path(`c(pwd)'/temp)
wbsrun using `c(pwd)'/temp/script.txt , time openbugs
type temp/log.txt
wbscoda using temp/out, clear openbugs
replace a0 = a0 - `m1'*a1 - `m2'*a2 - `m12'*a3
gen sd = 1/sqrt(tau)
mcmcstats a* sd 

pause "Press q to continue"
*===================================================================================================
*-------------------------------
* Section 8.3
* Dudong analysis
*-------------------------------
wbsdecode using data\dudongs.txt, clear
rename Y y
scatter y x , xtitle(Age) ytitle(Length)
* FIGURE 8.1  
graph export eps\dudongs.eps, replace
pause "Press q to continue"
*===================================================================================================
*-------------------------------
* Mata analysis for comparison
* Exercise 2 chapter 7
*-------------------------------
program drop _all
wbsdecode using data\dudongs.txt, clear
rename Y y
set matastrict off

mata:
mata clear
real scalar logpost(real matrix X,real rowvector b,real scalar ipar)
{
    mu = b[1]:-b[2]*b[4]:^X[,2]
	return( -0.5*b[3]*sum((X[,1]-mu):*(X[,1]-mu)) )
}
void aGibb(real matrix X,real rowvector b) 
{
    t = 27*b[3]+0.000001
    e = -b[2]*(b[4]:^X[,2])
	m = b[3]*sum((X[,1]-e))/t
	sd = 1/sqrt(t)
	b[1] = rnormal(1,1,m,sd)
}
void bGibb(real matrix X,real rowvector b) 
{
    e = -(b[4]:^X[,2])
	t = b[3]*sum(e:*e)+0.000001
	m = b[3]*sum(e:*(X[,1]:-b[1]))/t
	sd = 1/sqrt(t)
	b[2] = rnormal(1,1,m,sd)
}
void tGibb(real matrix X,real rowvector b) 
{
    mu = b[1]:-b[2]*(b[4]:^X[,2])
	gb = 1/(0.001+0.5*sum((X[,1]-mu):*(X[,1]-mu)))
	ga = 0.001+27/2
	b[3] = rgamma(1,1,ga,gb)
}
void sGibb(real scalar logp,pointer(function) lp,real matrix X,
   real rowvector b, real scalar ipar)
{
	if( ipar == 1 ) aGibb(X,b)
	else if( ipar == 2 ) bGibb(X,b)
	else if( ipar == 3 ) tGibb(X,b)
}
end

matrix b = 2, 1, 1, 0.75
matrix s = 1, 1, 1, 0.1

mcmcrun logpost Z b using temp/temp.csv, replace ///
	sampl( 3(sGibb) (mhstrnc, sd(s) lb(0.5) ub(1))  ) ///
	par(alpha beta tau gamma)  burn(1000) update(10000) adapt dots(0) ///
	mata data(b s Z=(y x)) 

import delimited temp/temp.csv , clear
gen sd = 1/sqrt(tau)
mcmcstats alpha beta gamma sd

pause "Press q to continue"
*===================================================================================================

*-------------------------------
* WinBUGS - default options
*-------------------------------
tomodel chapter8.do temp/model.txt V2
/*
model {
  for (i in 1:27) {
     y[i] ~ dnorm(mu[i],tau)
	 mu[i] <- alpha - beta*pow(gamma,x[i])
   }
   alpha ~ dnorm(0.0, 1.0E-6)
   beta  ~ dnorm(0.0, 1.0E-6)   
   gamma ~ dunif(0.5,1)
   tau   ~ dgamma(0.001,0.001)
}   
*/

wbsdecode using data\dudongs.txt, clear
rename Y y

wbslist (vector x y, format(%5.2f)) using temp/data.txt, replace 
wbslist (alpha=1,beta=1,gamma=0.9,tau=1) using temp/init.txt,  replace

wbsscript using temp/script.txt, model(model.txt) data(data.txt) ///
    inits(init.txt)  log(log.txt)   ///
    coda(out) set(alpha beta gamma tau ) ///
    burn(1000) update(10000) replace path(`c(pwd)'/temp) 
wbsrun using `c(pwd)'/temp/script.txt , time
type temp/log.txt

wbscoda using temp/out, clear 
gen sd = 1/sqrt(tau)
mcmcstats alpha beta gamma sd

mcmcac alpha , gopt(yscale(range(0 1)) ylabel(0(0.2)1))
graph save temp/plotw.gph , replace

pause "Press q to continue"
*===================================================================================================
*-------------------------------
* WinBUGS - over-relaxation
*-------------------------------

wbsdecode using data\dudongs.txt, clear
rename Y y

wbslist (vector x y, format(%5.2f)) using temp/data.txt, replace 
wbslist (alpha=1,beta=1,gamma=0.9,tau=1) using temp/init.txt,  replace

program drop _all
wbsscript using temp/script.txt, model(model.txt) data(data.txt) ///
    inits(init.txt)  log(log.txt)   ///
    coda(out) set(alpha beta gamma tau) overrelax ///
    burn(1000) update(10000) replace path(`c(pwd)'/temp)
wbsrun using `c(pwd)'/temp/script.txt , time
type temp/log.txt

wbscoda using temp/out, clear 
gen sd = 1/sqrt(tau)
mcmcstats alpha beta gamma sd

pause "Press q to continue"
*===================================================================================================
*-------------------------------
* OpenBUGS - default
*-------------------------------

wbsdecode using data\dudongs.txt, clear
rename Y y

wbslist (vector x y, format(%5.2f)) using temp/data.txt, replace 
wbslist (alpha=1,beta=1,gamma=0.9,tau=1) using temp/init.txt,  replace

program drop _all
wbsscript using temp/script.txt, model(model.txt) data(data.txt) ///
    inits(init.txt)  log(log.txt) ///
    coda(out) set(alpha beta gamma tau)  ///
    burn(1000) update(10000) replace openbugs path(`c(pwd)'/temp)
wbsrun using `c(pwd)'/temp/script.txt , openbugs time
type temp/log.txt

wbscoda using temp/out, clear openbugs
gen sd = 1/sqrt(tau)
mcmcstats alpha beta gamma sd

* FIGURE 8.1  
mcmctrace alpha beta gamma sd
graph export eps\dudong_trace.eps, replace
pause "Press q to continue"
*===================================================================================================
* FIGURE 8.2  
mcmccusum alpha beta gamma sd , ref(1)
graph export eps\dudong_cusum.eps, replace
pause "Press q to continue"
*===================================================================================================
* FIGURE 8.3  
scatter alpha gamma
graph export eps\dudong_scatter.eps, replace
pause "Press q to continue"
*===================================================================================================
mcmcac alpha , gopt(yscale(range(0 1)) ylabel(0(0.2)1))
graph save temp/ploto.gph , replace
graph combine temp/ploto.gph temp/plotw.gph
* FIGURE 8.4  
graph export eps\chap8_ac.eps, replace
pause "Press q to continue"
*===================================================================================================
*-------------------------------
* OpenBUGS - overrelaxed
*-------------------------------

wbsdecode using data\dudongs.txt, clear
rename Y y

wbslist (vector x y, format(%5.2f)) using temp/data.txt, replace 
wbslist (alpha=1,beta=1,gamma=0.9,tau=1) using temp/init.txt,  replace

program drop _all
wbsscript using temp/script.txt, model(model.txt) data(data.txt) ///
    inits(init.txt)  log(log.txt) ///
    coda(out) set(alpha beta gamma tau) overrelax ///
    burn(1000) update(10000) replace openbugs  path(`c(pwd)'/temp)
wbsrun using `c(pwd)'/temp/script.txt , openbugs time
type temp/log.txt

wbscoda using temp/out, clear openbugs
gen sd = 1/sqrt(tau)
mcmcstats alpha beta gamma sd

pause "Press q to continue"
*===================================================================================================

*-------------------------------
* JAW plot
*-------------------------------
use data/jaw.dta, clear
gen id = _n
stack id y1 id y2 id y3 id y4, into(id y) clear
rename _stack age
replace age = 7.5 + age/2
sort id age
scatter y age , c(a) ytitle(Jaw height) xtitle(Age)
* FIGURE 8.5  
graph export eps\jaw.eps, replace
pause "Press q to continue"
*===================================================================================================
*-------------------------------
* JAW Size analysis
* model in file modelj.txt
*-------------------------------
*-------------------------------
* WinBUGS - default
*-------------------------------
tomodel chapter8.do temp/model.txt V3
/*
model{
for (i in 1:20) {
y[i,1:4] ~ dmnorm(mu[1:4],Omega[1:4,1:4])
}
mu[1] <- beta0 + beta1*8
mu[2] <- beta0 + beta1*8.5
mu[3] <- beta0 + beta1*9
mu[4] <- beta0 + beta1*9.5
beta0 ~ dnorm(0, .01)
beta1 ~ dnorm(0.0, 0.001)
Omega[1:4,1:4]  ~ dwish(R[1:4,1:4],4)
}
*/

use data/jaw.dta, clear
matrix R = I(4)
wbslist (mat R) (structure y1 y2 y3 y4 , name(y) f(%5.1f) ) using temp/data.txt, replace

scalar beta0 = 40
scalar beta1 = 0
wbslist (scalar beta0 beta1 , format(%4.0f)) using temp/init.txt,replace

wbsscript using temp/script.txt, model(model.txt) data(data.txt) ///
    inits(init.txt)  log(log.txt) ///
    coda(jaw) set(beta0 beta1)  ///
    burn(1000) update(10000) replace path(`c(pwd)'/temp)

wbsrun using `c(pwd)'/temp/script.txt , time
wbscoda using temp/jaw, clear    
mcmcstats beta0 beta1   

pause "Press q to continue"
*===================================================================================================
   
*-------------------------------
* WinBUGS - overrelaxed
*-------------------------------

use data/jaw.dta, clear
matrix R = I(4)
wbslist (mat R) (structure y1 y2 y3 y4 , name(y) f(%5.1f) ) using temp/data.txt, replace

scalar beta0 = 40
scalar beta1 = 0
wbslist (scalar beta0 beta1 , format(%4.0f)) using temp/init.txt,replace

wbsscript using temp/script.txt, model(model.txt) data(data.txt) ///
    inits(init.txt)  log(log.txt) ///
    coda(jaw) set(beta0 beta1)  overrelax ///
    burn(1000) update(10000) replace path(`c(pwd)'/temp)

wbsrun using `c(pwd)'/temp/script.txt , time
wbscoda using temp/jaw, clear    
mcmcstats beta0 beta1   

pause "Press q to continue"
*===================================================================================================

*-------------------------------
* OpenBUGS - default
*-------------------------------

use data/jaw.dta, clear
matrix R = I(4)
wbslist (mat R) (structure y1 y2 y3 y4 , name(y) f(%5.1f) ) using temp/data.txt, replace

scalar beta0 = 40
scalar beta1 = 0
wbslist (scalar beta0 beta1) using temp/init.txt,replace

wbsscript using temp/script.txt, model(model.txt) data(data.txt) ///
    inits(init.txt)  log(log.txt) ///
    coda(jaw) set(beta0 beta1 Omega)  ///
    burn(1000) update(10000) replace openbugs path(`c(pwd)'/temp)

wbsrun using `c(pwd)'/temp/script.txt , openbugs time
type temp/log.txt
wbscoda using temp/jaw, clear openbugs   

mcmcac beta0 , gopt(yscale(range(0 1)) ylabel(0(0.2)1))
graph save temp/plotjo.gph, replace

matrix OMEGA = J(4,4,0)
forvalues j=1/4 {
   forvalues k=1/4 {
      qui gen v_`j'_`k' = .
      qui gen r_`j'_`k' = .
   }
}
forvalues i=1/10000 {
   forvalues j=1/4 {
      forvalues k=1/4 {
         matrix OMEGA[`j',`k'] = Omega_`j'_`k'[`i']
      }
   }
   matrix V = inv(OMEGA)
   forvalues j=1/4 {
      forvalues k=1/4 {
         qui replace v_`j'_`k' = V[`j',`k'] in `i'
      }
   }
   matrix R = corr(V)
   forvalues j=1/4 {
      forvalues k=1/4 {
         qui replace r_`j'_`k' = R[`j',`k'] in `i'
      }
   }
}
mcmcstats beta0 beta1 v_1_1 v_2_2 v_3_3 v_4_4 r_1_2 r_1_3 r_1_4 r_2_3 r_2_4 r_3_4

pause "Press q to continue"
*===================================================================================================

*-------------------------------
* OpenBUGS - overrelaxed
*-------------------------------

use data\jaw.dta, clear
matrix R = I(4)
wbslist (mat R) (structure y1 y2 y3 y4 , name(y) f(%5.1f) ) using temp/data.txt, replace

scalar beta0 = 40
scalar beta1 = 0
wbslist (scalar beta0 beta1) using temp/init.txt,replace

wbsscript using temp/script.txt, model(model.txt) data(data.txt) ///
    inits(init.txt)  log(log.txt) ///
    coda(jaw) set(beta0 beta1)  overrelax ///
    burn(1000) update(10000) replace openbugs path(`c(pwd)'/temp)

wbsrun using `c(pwd)'/temp/script.txt , openbugs time
type temp/log.txt
wbscoda using temp/jaw, clear openbugs   
mcmcstats beta0 beta1 , corr  

mcmcac beta0 , gopt(yscale(range(0 1)) ylabel(0(0.2)1))
graph save temp/plotjr.gph , replace

graph combine temp/plotjo.gph temp/plotjr.gph
* FIGURE 8.5 
graph export eps\chap8_or.eps, replace

pause "Press q to continue"
*===================================================================================================

*-------------------------------
* Section 8.7
* Prior sensitivity
*-------------------------------

use data/jaw.dta, clear
matrix R = I(4)
wbslist (mat R) (structure y1 y2 y3 y4 , name(y) f(%5.1f) ) using temp/data.txt, replace
scalar beta0 = 40
scalar beta1 = 0
wbslist (scalar beta0 beta1) using temp/init.txt,replace

wbsscript using temp/script.txt, model(model.txt) data(data.txt) ///
    inits(init.txt)  coda(jaw) set(beta0 beta1) ///
    burn(1000) update(10000) replace openbugs path(`c(pwd)'/temp)

tempname PF
postfile `PF' mu sd mn0 mn1 lb0 ub0 lb1 ub1 using temp/results.dta, replace
foreach mu of numlist 0 40 {
 foreach sd of numlist 10(10)100 {
   local precision = 1/(`sd'*`sd')
   tempname FH
   file open  `FH' using temp/model.txt, write replace
   file write `FH' "model {" _n
   file write `FH' "for (i in 1:20) {" _n
   file write `FH' "y[i,1:4] ~ dmnorm(mu[1:4],Omega[1:4,1:4])" _n
   file write `FH' "}" _n
   file write `FH' "mu[1] <- beta0 + beta1*8" _n
   file write `FH' "mu[2] <- beta0 + beta1*8.5" _n
   file write `FH' "mu[3] <- beta0 + beta1*9" _n
   file write `FH' "mu[4] <- beta0 + beta1*9.5" _n
   file write `FH' "beta0 ~ dnorm(`mu', `precision')" _n
   file write `FH' "beta1 ~ dnorm(0.0, 0.001)" _n
   file write `FH' "Omega[1:4,1:4]  ~ dwish(R[1:4,1:4],4)" _n
   file write `FH' "}" _n
   file close `FH'
   wbsrun using `c(pwd)'/temp/script.txt , openbugs
   wbscoda using temp/jaw, clear openbugs   
   mcmcstats beta0 beta1   
   post `PF' (`mu') (`sd') (r(mn1)) (r(mn2)) (r(lb1)) (r(ub1)) (r(lb2)) (r(ub2))
 }
}
postclose `PF'

use temp/results.dta, clear

replace sd = sd - 1 if mu == 0
replace sd = sd + 1 if mu == 40
summarize mn0 if sd > 55
local m = r(mean)
twoway (scatter mn0 sd) (rcap lb0 ub0 sd) , leg(off) ytitle(Estimate of {&beta}{subscript:0}) ///
  xtitle(Standard deviation of the prior on {&beta}{subscript:0}) yline(`m' ,lpat(dash))
* FIGURE 8.5 
graph export eps\simbeta0.eps, replace
summarize mn1 if sd > 55
local m = r(mean)
twoway (scatter mn1 sd) (rcap lb1 ub1 sd) , leg(off) ytitle(Estimate of {&beta}{subscript:1}) ///
  xtitle(Standard deviation of the prior on {&beta}{subscript:0}) yline(`m' ,lpat(dash))
* FIGURE 8.5 
graph export eps\simbeta1.eps, replace

pause "Press q to continue"
*===================================================================================================
